@@ -1,8 +1,7 @@
 #ifndef TIMEWIDGET_IMPL_H
 #define TIMEWIDGET_IMPL_H
-
 /***************************************************************************
-    copyright            : (C) 2005 by Carsten Niehaus
+    copyright            : (C) 2006 by Carsten Niehaus
     email                : cniehaus@kde.org
  ***************************************************************************/
 /***************************************************************************
@@ -14,28 +13,30 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "timewidget.h"
+#include <QWidget>
+#include "ui_timewidget.h"
+
+class Element;
 
 /**
  * @author Carsten Niehaus
  */
-class TimeWidgetIMPL : public TimeWidget
+class TimeWidgetImpl : public QWidget, public Ui_TimeWidget
 {
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		/**
-		 * @param parent The parent of this widget
-		 * @param name The name of this widget
-		 */
-		TimeWidgetIMPL( QWidget *parent = 0, const char* name = 0 );
+    public:
+        /**
+         * @param parent The parent of this widget
+         */
+        TimeWidgetImpl( QWidget *parent = 0 );
 
-	public slots:
-		/**
-		 * this slot will react to the new year
-		 * @param value the new year
-		 */
-		void slotChanged( int value );
+//X     private:
+//X         QList<Element*> m_list;
+//X         QString m_htmlBegin;
+//X         QString m_htmlEnd;
+
+//X     private slots:
+//X         void setNewTime( int newtime );
 };
 #endif // TIMEWIDGET_IMPL_H
-
