@@ -15,8 +15,8 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 // 02110-1301, USA.
 
-#ifndef KIG_OBJECTS_POLIGON_TYPE_H
-#define KIG_OBJECTS_POLIGON_TYPE_H
+#ifndef KIG_OBJECTS_POLYGON_TYPE_H
+#define KIG_OBJECTS_POLYGON_TYPE_H
 
 #include "base_type.h"
 
@@ -93,6 +93,11 @@ public:
   const Coordinate moveReferencePoint( const ObjectTypeCalcer& o ) const;
 };
 
+int polygonlineintersection( const std::vector<Coordinate>& ppoints, 
+      const Coordinate a, const Coordinate b,
+      bool boundleft, bool boundright, double& t1, double& t2,
+      std::vector<Coordinate>::const_iterator& intersectionside );
+
 class PolygonLineIntersectionType
   : public ArgsParserObjectType
 {
@@ -104,6 +109,17 @@ public:
   const ObjectImpType* resultId() const;
 };
 
+class PolygonPolygonIntersectionType
+  : public ArgsParserObjectType
+{
+  PolygonPolygonIntersectionType();
+  ~PolygonPolygonIntersectionType();
+public:
+  static const PolygonPolygonIntersectionType* instance();
+  ObjectImp* calc( const Args& parents, const KigDocument& ) const;
+  const ObjectImpType* resultId() const;
+};
+
 class PolygonVertexType
   : public ArgsParserObjectType
 {
@@ -136,4 +152,5 @@ public:
   ObjectImp* calc( const Args& parents, const KigDocument& ) const;
   const ObjectImpType* resultId() const;
 };
+
 #endif
