@@ -28,6 +28,7 @@
 #include <QPainter>
 #include <QTextStream>
 #include <QDomDocument>
+#include <QWidget>
 
 //KDE includes
 #include <KAction>
@@ -57,11 +58,13 @@
 #include "langutils.h"
 #include "kltheme.h"
 
-class fontsdlg : public QDialog, public Ui::fontsdlg
+class fontsdlg : public QWidget, public Ui::fontsdlg
 {    
     public:
-        fontsdlg( QWidget * parent ) : QDialog(parent)
-        { setupUi(this); }
+        fontsdlg( QWidget * parent ) : QWidget(parent)
+        {
+            setupUi(this);
+        }
 };
 
 const int ID_KIDB      = 100;
@@ -244,7 +247,7 @@ void KLettres::loadSettings()
     }
     QString langString = LangUtils::getLanguagesNames(LangUtils::getLanguages())[LangUtils::getLanguages().indexOf(Prefs::language())];
     m_languageAction->setCurrentItem(LangUtils::getLanguages().indexOf(Prefs::language()));
-    langString.replace("&", QString());
+    langString.remove('&');
     m_langLabel->setText(langString);
     loadLangToolBar();
     // load default level
@@ -334,7 +337,7 @@ void KLettres::slotChangeLanguage(int newIndex)
     Prefs::self()->writeConfig();
     // Update the StatusBar
     QString langString = LangUtils::getLanguagesNames(LangUtils::getLanguages())[newIndex];
-    langString.replace("&", QString());
+    langString.remove('&');
     m_langLabel->setText(langString);
     loadLangToolBar();
     // Change language effectively
@@ -422,7 +425,7 @@ void KLettres::loadLangToolBar()
         QTextStream readFileStr(&openFileStream);
         readFileStr.setCodec("UTF-8");
         //allData contains all the words from the file
-        allData = readFileStr.readAll().split("\n");
+        allData = readFileStr.readAll().split('\n');
         openFileStream.close();
         for (int i=0; i<(int) allData.count(); ++i) {
             if (!allData[i].isEmpty()) {
