@@ -25,6 +25,7 @@
 #include <kmenu.h>
 #include <kstatusbar.h>
 #include <ktip.h>
+#include <kled.h>
 #include <kmessagebox.h>
 #include <kstandardaction.h>
 #include <kstandarddirs.h>
@@ -44,11 +45,14 @@
 #include "ksutils.h"
 #include "ksnumbers.h"
 #include "infoboxes.h"
-#include "indimenu.h"
 #include "simclock.h"
 #include "widgets/timestepbox.h"
 
-#include "config-kstars.h"
+#include <config-kstars.h>
+
+#ifdef HAVE_INDI_H
+#include "indimenu.h"
+#endif
 
 //This file contains functions that kstars calls at startup (except constructors).
 //These functions are declared in kstars.h
@@ -70,7 +74,7 @@ void KStars::initActions() {
 //     ka->setShortcuts( KShortcut( Qt::CTRL+Qt::Key_W ) );
 //     connect( ka, SIGNAL( triggered() ), this, SLOT( closeWindow() ) );
 
-    QAction *ka = KNS::standardAction(i18n("Download New Data..."), this, SLOT(slotDownload()), actionCollection(), "get_data");
+    KAction *ka = KNS::standardAction(i18n("Download New Data..."), this, SLOT(slotDownload()), actionCollection(), "get_data");
     ka->setShortcuts( KShortcut( Qt::CTRL+Qt::Key_D ) );
     ka->setWhatsThis(i18n("Downloads new data"));
     ka->setToolTip(ka->whatsThis());
@@ -115,8 +119,7 @@ void KStars::initActions() {
     ta->setIcon( KIcon( "media-playback-pause" ) );
     ta->setText( i18n( "Stop &Clock" ) );
     QObject::connect( ta, SIGNAL( triggered() ), this, SLOT( slotToggleTimer() ) );
-    QObject::connect(data()->clock(), SIGNAL(clockStarted()), ta, SLOT(slotToggled(false)) );
-    QObject::connect(data()->clock(), SIGNAL(clockStopped()), ta, SLOT(slotToggled(true)) );
+    QObject::connect(data()->clock(), SIGNAL(clockToggled(bool)), ta, SLOT(setChecked(bool)) );
     //UpdateTime() if clock is stopped (so hidden objects get drawn)
     QObject::connect(data()->clock(), SIGNAL(clockStopped()), this, SLOT(updateTime()) );
 
@@ -354,6 +357,11 @@ void KStars::initActions() {
     ka->setShortcuts( KShortcut(Qt::CTRL+Qt::Key_U ) );
     connect( ka, SIGNAL( triggered() ), this, SLOT( slotWUT() ) );
 
+    ka = actionCollection()->addAction( "skycalendar");
+    ka->setText( i18n( "Sky Calendar...") );
+//    ka->setShortcuts( KShortcut(Qt::CTRL+Qt::Key_H ) );
+    connect( ka, SIGNAL( triggered() ), this, SLOT( slotCalendar() ) );
+
 //FIXME: implement glossary
 //     ka = actionCollection()->addAction( "glossary");
 //     ka->setText( i18n( "Glossary...") );
@@ -376,14 +384,17 @@ void KStars::initActions() {
     connect( ka, SIGNAL( triggered() ), this, SLOT( slotJMoonTool() ) );
 
     // devices Menu
+#ifdef HAVE_INDI_H
 #ifndef Q_WS_WIN
+
     ka = actionCollection()->addAction( "telescope_wizard");
     ka->setText( i18n("Telescope Wizard...") );
     connect( ka, SIGNAL( triggered() ), this, SLOT( slotTelescopeWizard() ) );
 
-    ka = actionCollection()->addAction( "telescope_properties");
-    ka->setText( i18n("Telescope Properties...") );
-    connect( ka, SIGNAL( triggered() ), this, SLOT( slotTelescopeProperties() ) );
+    // TODO redesign after KDE 4.2 is released
+    //ka = actionCollection()->addAction( "telescope_properties");
+    //ka->setText( i18n("Telescope Properties...") );
+    //connect( ka, SIGNAL( triggered() ), this, SLOT( slotTelescopeProperties() ) );
 
     ka = actionCollection()->addAction( "device_manager");
     ka->setText( i18n("Device Manager...") );
@@ -402,6 +413,7 @@ void KStars::initActions() {
     ka = actionCollection()->addAction( "configure_indi");
     ka->setText( i18n("Configure INDI...") );
     connect( ka, SIGNAL( triggered() ), this, SLOT( slotINDIConf() ) );
+#endif
 #endif
 
     //Help Menu:
@@ -481,10 +493,11 @@ void KStars::initActions() {
     ta->setToolTip( i18n("Toggle opaque ground") );
     connect( ta, SIGNAL( triggered() ), this, SLOT( slotViewToolBar() ) );
 
+
     setXMLFile( "kstarsui.rc" );
 
-    if (Options::fitsSaveDirectory().isEmpty())
-        Options::setFitsSaveDirectory(QDir:: homePath());
+    if (Options::fitsDir().isEmpty())
+        Options::setFitsDir(QDir:: homePath());
 }
 
 void KStars::initFOV() {
@@ -519,7 +532,7 @@ void KStars::initFOV() {
         QTextStream stream( &f );
         while ( !stream.atEnd() ) {
             QString line = stream.readLine();
-            fields = line.split( ":" );
+            fields = line.split( ':' );
 
             if ( fields.count() == 4 ) {
                 nm = fields[0].trimmed();
@@ -547,7 +560,7 @@ void KStars::initFOV() {
 void KStars::initStatusBar() {
     statusBar()->insertPermanentItem( i18n( " Welcome to KStars " ), 0, 1 );
     statusBar()->setItemAlignment( 0, Qt::AlignLeft | Qt::AlignVCenter );
-
+    
     QString s = "000d 00m 00s,   +00d 00\' 00\""; //only need this to set the width
     if ( Options::showAltAzField() ) {
         statusBar()->insertPermanentFixedItem( s, 1 );
@@ -561,6 +574,10 @@ void KStars::initStatusBar() {
         statusBar()->changeItem( QString(), 2 );
     }
 
+    m_DiskLED = new KLed( Qt::red, statusBar() );
+    m_DiskLED->off();
+    statusBar()->addPermanentWidget( m_DiskLED );
+
     if ( ! Options::showStatusBar() ) statusBar()->hide();
 }
 
@@ -595,8 +612,11 @@ void KStars::datainitFinished(bool worked) {
     connect( TimeStep, SIGNAL( scaleChanged( float ) ), this,
              SLOT( mapGetsFocus() ) );
 
+
+    #ifdef HAVE_INDI_H
     //Initialize INDIMenu
     indimenu = new INDIMenu(this);
+    #endif
 
     //Initialize Observing List
     obsList = new ObservingList( this );
