@@ -32,7 +32,7 @@
 #include <QPixmap>
 #include <kglobal.h>
 
-#define KSTARS_VERSION "1.4.2"
+#define KSTARS_VERSION "1.5.0"
 
 static const char description[] =
     I18N_NOOP("Desktop Planetarium");
@@ -178,7 +178,7 @@ int main(int argc, char *argv[])
 
     if ( ! datestring.isEmpty() && ! KStarsDateTime::fromString( datestring ).isValid() ) {
         kWarning() << i18n( "Using CPU date/time instead." ) ;
-        datestring = QString();
+        datestring.clear();
     }
 
     new KStars( true, ! args->isSet( "paused" ), datestring );
