@@ -96,7 +96,7 @@ void DeepSkyComponent::init(KStarsData *data)
         else if ( iflag == 'N' ) cat = "NGC";
 
         ingc = line.mid( 1, 4 ).toInt();  // NGC/IC catalog number
-        if ( ingc==0 ) cat = QString(); //object is not in NGC or IC catalogs
+        if ( ingc==0 ) cat.clear(); //object is not in NGC or IC catalogs
 
         //coordinates
         rah = line.mid( 6, 2 ).toInt();
@@ -109,7 +109,7 @@ void DeepSkyComponent::init(KStarsData *data)
 
         //B magnitude
         ss = line.mid( 23, 4 );
-    if (ss == "    " ) { mag = 99.9; } else { mag = ss.toFloat(); }
+    if (ss == "    " ) { mag = 99.9f; } else { mag = ss.toFloat(); }
 
         //object type
         type = line.mid( 29, 1 ).toInt();
@@ -139,7 +139,7 @@ void DeepSkyComponent::init(KStarsData *data)
         //Messier number
         if ( line.mid( 70,1 ) == "M" ) {
             cat2 = cat;
-            if ( ingc==0 ) cat2 = QString();
+            if ( ingc==0 ) cat2.clear();
             cat = "M";
             imess = line.mid( 72, 3 ).toInt();
         }
@@ -167,7 +167,7 @@ void DeepSkyComponent::init(KStarsData *data)
                 snum.setNum( ingc );
                 name2 = cat2 + ' ' + snum;
             } else {
-                name2 = QString();
+                name2.clear();
             }
         }
         else {
@@ -231,7 +231,7 @@ void DeepSkyComponent::mergeSplitFiles() {
     QString firstFile = KStandardDirs::locateLocal("appdata", "ngcic01.dat");
     if ( ! QFile::exists( firstFile ) ) return;
     QDir localDir = QFileInfo( firstFile ).absoluteDir();
-    QStringList catFiles = localDir.entryList( "ngcic??.dat" );
+    QStringList catFiles = localDir.entryList( QStringList( "ngcic??.dat" ) );
 
     kDebug() << "Merging split NGC/IC files" << endl;
 
@@ -390,10 +390,10 @@ void DeepSkyComponent::drawDeepSkyCatalog( QPainter& psky, bool drawObject,
                     // most objects don't have those, so we only change colors temporarily
                     // for the few exceptions. Changing color is expensive!!!
                     bool bColorChanged = false;
-                    if ( obj->isCatalogM() && obj->ImageList.count() > 1 ) {
+                    if ( obj->isCatalogM() && obj->ImageList().count() > 1 ) {
                         psky.setPen( colorExtra );
                         bColorChanged = true;
-                    } else if ( (!obj->isCatalogM()) && obj->ImageList.count() ) {
+                    } else if ( (!obj->isCatalogM()) && obj->ImageList().count() ) {
                         psky.setPen( colorExtra );
                         bColorChanged = true;
                     }
