@@ -66,76 +66,70 @@ void Executer::initialize(TreeNode* tree, ErrorList* _errorList)
 
 void Executer::execute()
 {
+	//Do we have to do anything?
 	if (finished || waiting) return;
-// 	qDebug() << "Executer::execute() -- main execute loop called";
 
-	if (executeCurrent) {
+	if(executeCurrent) {
 		// executeCurrent is used to make sure the currentNode will be executed
 		// this way the tree will not be walked before the execution...
-// 		qDebug() << ">> jumping to tree node, and executing: " << currentNode->token()->look();
 		executeCurrent = false;
 		execute(currentNode);
 		return;
-	} else if (returning) {
-		delete functionStack.top().variableTable;
-		currentNode = functionStack.pop().function;
-// 		qDebug() << ">> returning to: " << currentNode->token()->look();
+	}
 
-		if (returnValue == 0) currentNode->setNullValue();  // Handle an empty return value
-		else currentNode->setValue(returnValue);
-// // // 		returnValue = 0;  // DONT DO THIS... the the functionCall node that get recalled now handle it...
-// // // 		returning   = false;
+	if(returning) {
+		//We are returning from a function call
+		
+		// Take the last called function from the function stack
+		CalledFunction calledFunction = functionStack.pop();
+		
+		// Delete the local variables of the called function
+		delete calledFunction.variableTable;
+		currentNode = calledFunction.function;
+
+		if (returnValue == 0)
+			currentNode->setNullValue(); // Handle an empty return value
+		else
+			currentNode->setValue(returnValue);
 
 		execute(currentNode);
 		return;
-	} else {
-		if (newScope == 0) {  // same scope
-// 			qDebug() << ">> to next sibling";
-			
-			TreeNode* currentParent = currentNode->parent();
-			currentNode = currentNode->nextSibling();
-		
-			if (currentNode == 0) {  // running off sibling list
-// 				qDebug() << ">> to parent";
-				currentNode = currentParent;
-				if (currentNode == rootNode) {
-// 					qDebug() << ">> hit root";
-					finished = true;
-					return;
-				}
+	}
 
-				// don't report scopes (their are not really executed)
-				if (currentNode->token()->type() != Token::Scope)
-					std::cout << "EXE> " << qPrintable(currentNode->token()->look()) << std::endl;
-				execute(currentNode);
+	if(newScope == 0) {
+		TreeNode* currentParent = currentNode->parent();
+		currentNode = currentNode->nextSibling();
+		
+		if(currentNode == 0) { //running off sibling list
+			currentNode = currentParent;
+			
+			if(currentNode == rootNode) {
+				finished = true;
 				return;
 			}
-		} else {  // entering new scope
-// 			qDebug() << ">> to the new scope";
-			currentNode = newScope;
-			newScope = 0;
-			
-			// skip ahead to frist child (if any) else we will not get into the scope
-			if (currentNode->hasChildren()) currentNode = currentNode->firstChild();
+
+			printExe();
+
+			execute(currentNode);
+			return;
 		}
-	}
-	
-// 	qDebug() << ">> to top";
-	while (currentNode->hasChildren() && currentNode->token()->type() != Token::Scope) {
-//Niels: I don't know why, but the following lines break learn
-//		if (currentNode->token()->type() == Token::Learn) {
-//			currentNode = currentNode->nextSibling();
-//			if (currentNode == 0) {
-//				finished = true;
-//				return;
-//			}
-//		}
-		currentNode = currentNode->firstChild();
+
+	}else{
+		// We're entering a new scope
+		// skip ahead to frist child (if any) else we will not get into the scope
+		if(newScope->hasChildren())
+			currentNode = newScope->firstChild();
+		else
+			currentNode = newScope;
+
+		newScope = 0;
 	}
 
-	// don't report scopes (their are not really executed)
-	if (currentNode->token()->type() != Token::Scope)
-		std::cout << "EXE> " << qPrintable(currentNode->token()->look()) << std::endl;
+
+	while (currentNode->hasChildren() && currentNode->token()->type() != Token::Scope)
+		currentNode = currentNode->firstChild();
+	
+	printExe();
 
 	execute(currentNode);
 }
@@ -282,9 +276,9 @@ bool Executer::checkParameterQuantity(TreeNode* node, uint quantity, int errorCo
 	
 	if (nodeSize != quantity) {
 		if (nodeSize < quantity) {
-			addError(i18n("The %1 command was called with %2 but needs 1 parameter.", node->token()->look(), nodeSize), *node->token(), errorCode);
+			addError(i18np("The %2 command was called with %3 but needs 1 parameter.", "The %2 command was called with %3 but needs %1 parameters.", quantity, node->token()->look(), nodeSize), *node->token(), errorCode);
 		} else {
-			addError(i18n("The %1 command was called with %2 but only accepts 1 parameter.", node->token()->look(), nodeSize), *node->token(), errorCode);
+			addError(i18np("The %2 command was called with %3 but only accepts 1 parameter.", "The %2 command was called with %3 but only accepts %1 parameters.", quantity, node->token()->look(), nodeSize), *node->token(), errorCode);
 		}
 		return false;
 	}
@@ -479,8 +473,14 @@ void Executer::executeElse(TreeNode* node) {
 }
 void Executer::executeRepeat(TreeNode* node) {
 //	qDebug() << "Executer::executeRepeat()";
-	breaking = false;
 	QString id = QString("__%1_%2").arg(node->token()->look()).arg((long)node);
+
+	if(breaking) {
+		breaking = false;
+		currentVariableTable()->remove(id);
+		return;
+	}
+
 	// the iteration state is stored on the variable table
 	if (currentVariableTable()->contains(id)) {
 		int currentCount = ROUND2INT((*currentVariableTable())[id].number());
@@ -506,6 +506,14 @@ void Executer::executeWhile(TreeNode* node) {
 	//     exec scope, exec expression, exec scope, exec expression, ...
 
 	QString id = QString("__%1_%2").arg(node->token()->look()).arg((long)node);
+
+	if (breaking) {
+		// We hit a break command while executing the scope
+		breaking = false; // Not breaking anymore
+		currentVariableTable()->remove(id); // remove the value (cleanup)
+		return; // Move to the next sibbling
+	}
+
 	if (currentVariableTable()->contains(id)) {
 		newScope = node; // re-execute the expression
 		currentVariableTable()->remove(id);
@@ -529,19 +537,35 @@ void Executer::executeForTo(TreeNode* node) {
 	// after one iteration the expression is not re-executed.
 	// so we do: exec scope, exec expressions, exec scope, exec expressions, ...
 
+	//TODO: We have the cleanup part twice (after breaking and after the last iteration
+	// perhaps clean it up by putting in in one place?
+
 	bool firstIteration = false;
 	if (functionStack.isEmpty() || functionStack.top().function != node) {
 		// if this for loop is called for the first time...
 		CalledFunction c;
 		c.function      = node;
-		c.variableTable = new VariableTable();
+		// TODO: Find a better solution then this for nested for loops
+		//c.variableTable = new VariableTable();
+		c.variableTable = currentVariableTable();
 		functionStack.push(c);
 
 		currentVariableTable()->insert(node->child(0)->token()->look(), Value(node->child(1)->value()->number()));
 		firstIteration = true;
 	}
 
-	QString id = QString("__%1_%2").arg(node->token()->look()).arg((long)node);
+        QString id = QString("__%1_%2").arg(node->token()->look()).arg((long)node);
+
+	if(breaking) {
+		breaking = false;
+		//delete functionStack.top().variableTable;
+		functionStack.pop();
+		// if we don't delete the functionStack's varibleTable any more
+		// do remove the for loops id..
+		currentVariableTable()->remove(id);
+		return;
+	}
+
 
 	if (currentVariableTable()->contains(id)) {
 		newScope = node; // re-execute the expressions
@@ -565,13 +589,35 @@ void Executer::executeForTo(TreeNode* node) {
 		newScope = node->child(4); // (re-)execute the scope
 	} else {
 		// cleaning up after last iteration...
-		delete functionStack.top().variableTable;
+		//delete functionStack.top().variableTable;
 		functionStack.pop();
+		// if we don't delete the functionStack's varibleTable any more
+		// do remove the for loops id..
+		currentVariableTable()->remove(id);
 	}
 }
 void Executer::executeBreak(TreeNode* node) {
 //	qDebug() << "Executer::executeBreak()";
 	if (!checkParameterQuantity(node, 0, 20000+Token::Break*100+90)) return;
+
+	breaking = true;
+
+	// Check for the first parent which is a repeat, while of for loop.
+	// If found, switch the newScope to them so they can break.
+	QList<int> tokenTypes;
+	tokenTypes.append(Token::Repeat);
+	tokenTypes.append(Token::While);
+	tokenTypes.append(Token::ForTo);
+
+	TreeNode* ns = getParentOfTokenTypes(node, &tokenTypes);
+
+	if(ns!=0)
+		newScope = ns;
+	//else
+		// We could add an error right HERE
+	
+	// At the moment we just ignore a break when we couldn't 
+	// find a matching parent
 }
 void Executer::executeReturn(TreeNode* node) {
 //	qDebug() << "Executer::executeReturn()";
@@ -895,8 +941,6 @@ void Executer::executeRandom(TreeNode* node) {
 	if (!checkParameterQuantity(node, 2, 20000+Token::Random*100+90)) return;
 	TreeNode* nodeX = node->child(0);  // getting
 	TreeNode* nodeY = node->child(1);
-	execute(nodeX);  // executing
-	execute(nodeY);
 	
 	if (!checkParameterType(node, Value::Number, 20000+Token::Random*100+91)) return;
 	double x = nodeX->value()->number();
@@ -1006,5 +1050,19 @@ void Executer::executeRound(TreeNode* node) {
 
 //END GENERATED executer_cpp CODE
 
+TreeNode* Executer::getParentOfTokenTypes(TreeNode* child, QList<int>* types) {
+	foreach(int type, *types) {
+		if(child->parent()->token()->type()==type)
+			return child->parent();
+		else if(child->parent()->token()->type()==Token::Root)
+			return 0;
+	}
+	return getParentOfTokenTypes(child->parent(), types);
+}
+
+void Executer::printExe() {
+	if (currentNode->token()->type() != Token::Scope)
+		std::cout << "EXE> " << qPrintable(currentNode->token()->look()) << std::endl;
+}
 
 #include "executer.moc"
