@@ -44,6 +44,10 @@
 #include <ktoolbarlabelaction.h>
 #include <ktoolbarpopupaction.h>
 #include <kdeprintdialog.h>
+#include <kfiledialog.h>
+#include <ksavefile.h>
+#include <kio/netaccess.h>
+#include <KTabWidget>
 
 #include "interpreter/errormsg.h"
 #include "interpreter/translator.h"
@@ -170,7 +174,7 @@ void MainWindow::setupActions()
 	//TODO WHATISTHIS to each action
 	//Similar to a status tip, but not the same.
 	//A status tip is displayed on hover, a whatisthis is displayed when
-	//an item is clicked on it whatisthis mode  (cies doesnt like whatis mode for every litlle widget)
+	//an item is clicked on it whatisthis mode  (cies doesn't like whatis mode for every little widget)
 
 	// File menu actions
 	a = actionCollection()->addAction(KStandardAction::New,  "file_new", editor, SLOT(newFile()));
@@ -195,6 +199,12 @@ void MainWindow::setupActions()
 	a->setStatusTip(i18n("Save the current file under a different name"));
 	a->setWhatsThis(i18n("Save File As: Save the current file under a different name"));
 
+	saveAsPictureAct  = actionCollection()->addAction("save_as_picture");
+	saveAsPictureAct->setText(i18n("Save &as Picture..."));
+	saveAsPictureAct->setStatusTip(i18n("Save the current canvas as a picture"));
+	saveAsPictureAct->setWhatsThis(i18n("Save as Picture: Save the current canvas as a picture"));
+	connect(saveAsPictureAct, SIGNAL(triggered()), this, SLOT(saveAsPicture()));
+
 	runAct  = new KAction(KIcon("media-playback-start"), i18n("&Run"), this);
 	actionCollection()->addAction("run", runAct );
 	runAct->setShortcut(QKeySequence(Qt::Key_F5));
@@ -362,6 +372,12 @@ void MainWindow::setupActions()
 	connect(console, SIGNAL(execute(const QString&)), this, SLOT(execute(const QString&)));
 
 
+	QAction *executeConsole = actionCollection()->addAction("execute_console");
+	executeConsole->setIcon(KIcon("go-jump-locationbar"));
+	executeConsole->setText(i18n("Execute"));
+	connect(executeConsole, SIGNAL(triggered()), console, SLOT(executeActionTriggered()));
+	executeConsole->setWhatsThis(i18n("Execute: Executes the current line in the console"));
+
 	// Help menu actions
 	//TODO: implement context help
 	contextHelpAct = ac->addAction("context_help");
@@ -449,7 +465,7 @@ void MainWindow::setupCanvas()
 	QHBoxLayout* centralLayout = new QHBoxLayout(centralWidget);
 	centralLayout->setMargin(0); // MARGIN_SIZE);
 
-	canvasTabWidget = new QTabWidget(this);
+	canvasTabWidget = new KTabWidget(this);
 
 	canvasTab = new QWidget();
 	QHBoxLayout* canvasLayout = new QHBoxLayout(canvasTab);
@@ -591,7 +607,7 @@ void MainWindow::updateLanguagesMenu()
 	QActionGroup* languageGroup = new QActionGroup(this);
 	connect(languageGroup, SIGNAL(triggered(QAction *)), this, SLOT(setLanguage(QAction *)));
 	QAction* a;
-	// sort the dictionaries using an algorithm found the the qt docs:
+	// sort the dictionaries using an algorithm found in the qt docs:
 	QMap<QString, QString> map;
 	foreach (const QString &lang_code, KGlobal::locale()->languageList())
 		map.insert(codeToFullName(lang_code), lang_code);
@@ -665,7 +681,7 @@ void MainWindow::updateOnCursorPositionChange(int row, int col, const QString& l
 	statusBarPositionLabel->setText(i18n(" Line: %1 Column: %2 ", row, col));
 
 	Token* cursorToken = editor->currentToken(line, col);
-	QString desc = QString();
+	QString desc;
 	if (cursorToken != 0) {
 		QString look = cursorToken->look();
 		int cat = cursorToken->category();
@@ -717,6 +733,7 @@ bool MainWindow::setCurrentLanguage(const QString &lang_code)  // 2 or 5 digit c
 		currentLanguageCode = lang_code;
 		statusBarLanguageLabel->setText(' ' + codeToFullName(lang_code) + ' ');
 		updateExamplesMenu();
+		editor->rehighlight();
 		result = true;
 	} else {
 		KMessageBox::error(this, i18n("Could not change the language to %1.", codeToFullName(lang_code)));
@@ -783,7 +800,7 @@ QString MainWindow::execute(const QString &operation)
 	pauseAct->setEnabled(false);
 	abortAct->setEnabled(false);
 
-	QString errorMessage = QString();
+	QString errorMessage;
 
 	if (interpreter->encounteredErrors()) {
 		ErrorList* errorList = interpreter->getErrorList();
@@ -926,6 +943,25 @@ void MainWindow::slotMessageDialog(const QString& text)
 	run();
 }
 
+void MainWindow::saveAsPicture()
+{
+	//Copied from edit code for file selection
+	KUrl url = KFileDialog::getSaveUrl(QString(), QString("*.png|%1\n*|%2").arg(i18n("PNG Images")).arg(i18n("All files")), this, i18n("Save as Picture"));
+	if (url.isEmpty()) return;
+	if (KIO::NetAccess::exists(url, KIO::NetAccess::SourceSide, this) &&
+		KMessageBox::warningContinueCancel(this,
+			i18n("Are you sure you want to overwrite %1?", url.fileName()),
+			i18n("Overwrite Existing File"),KGuiItem(i18n("&Overwrite")),
+			KStandardGuiItem::cancel(),
+			i18n("&Overwrite")
+			) != KMessageBox::Continue
+		) return;
+	//Get our image from the canvas
+	QImage pict = canvas->getPicture();
+	//Save as png
+	pict.save(url.path(), "PNG");
+}
+
 // END
 
 #include "mainwindow.moc"
