@@ -50,6 +50,7 @@
 #include <KProcess>
 #include <KTemporaryFile>
 #include <kwindowsystem.h>
+#include <kio/netaccess.h>
 
 #include "keduvocdocument.h"
 #include "keduvoclesson.h"
@@ -311,7 +312,7 @@ void KWordQuizApp::initActions()
 
   quizQA = actionCollection()->addAction("quiz_qa");
   quizQA->setIcon(KIcon("qa"));
-  quizQA->setText(i18n("&Question && Answer"));
+  quizQA->setText(i18n("&Question and Answer"));
   quizQA->setIconText(i18n("Q&&A"));
   quizQA->setShortcut(QKeySequence(Qt::Key_F9));
   quizQA->setWhatsThis(i18n("Starts a question and answer session using the active vocabulary"));
@@ -406,7 +407,7 @@ void KWordQuizApp::initActions()
   updateSpecialCharIcons();
 
   if (!initialGeometrySet())
-    resize( QSize(650, 500).expandedTo(minimumSizeHint()));
+    resize( QSize(650, 450).expandedTo(minimumSizeHint()));
   setupGUI(ToolBar | Keys | StatusBar | Create);
   setAutoSaveSettings();
 
@@ -568,7 +569,7 @@ void KWordQuizApp::openUrl(const KUrl& url)
 void KWordQuizApp::openDocumentFile(const KUrl& url)
 {
   slotStatusMsg(i18n("Opening file..."));
-  if (!url.isEmpty()) {
+  if (!url.isEmpty() && KIO::NetAccess::exists(url, KIO::NetAccess::SourceSide, this)) {
     int result = m_doc->open(url);
     if (result == KEduVocDocument::NoError) {
       m_tableModel->reset();
@@ -888,7 +889,7 @@ void KWordQuizApp::slotFileClose()
       slotQuizEditor();
       QAction *a = actionCollection()->action(QString("mode_%1").arg(QString::number(Prefs::mode())));
       slotModeActionGroupTriggered(a);
-      m_tableView ->setFocus();
+      m_tableView ->selectionModel()->setCurrentIndex(m_sortFilterModel->index(0, 0), QItemSelectionModel::SelectCurrent);
       m_undoStack->clear();
     }
 
