@@ -1,5 +1,5 @@
 /* This file is part of KWordQuiz
-  Copyright (C) 2004 Peter Hedlund <peter.hedlund@kdemail.net>
+  Copyright (C) 2004-2008 Peter Hedlund <peter.hedlund@kdemail.net>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -19,31 +19,28 @@
 #ifndef WQPRINTDIALOGPAGE_H
 #define WQPRINTDIALOGPAGE_H
 
+#include <QtGui/QGroupBox>
+#include <QtGui/QButtonGroup>
 
-#include <kdeprint/kprintdialogpage.h>
+#include "prefs.h"
 
 /**
 Print dialog page to provide specific print options for KWordQuiz
 
 @author Peter Hedlund
 */
-class WQPrintDialogPage : public KPrintDialogPage
+class WQPrintDialogPage : public QWidget
 {
 Q_OBJECT
 public:
-    WQPrintDialogPage( QWidget *parent = 0, const char *name = 0 );
+  WQPrintDialogPage(QWidget *parent = 0);
 
-    ~WQPrintDialogPage();
-   //reimplement virtual functions
-   void getOptions(QMap<QString,QString>& opts, bool incldef = false);
-   void setOptions(const QMap<QString,QString>& opts);
-   bool isValid(const QString& msg );
+  int printStyle();
+  void setPrintStyle(int style);
 
 private:
-  QButtonGroup * g;
-  QRadioButton * rb0;
-  QRadioButton * rb1;
-  QRadioButton * rb2;
+  QGroupBox *g;
+  QButtonGroup *bg;
 };
 
 #endif
