@@ -11,6 +11,7 @@
 
 #include "QtMainWindow.h"
 
+#include <QtCore/QDebug>
 #include <QtCore/QSettings>
 #include <QtGui/QCloseEvent>
 
@@ -43,6 +44,8 @@ namespace
     const char* DISTANCE_STRING = "Altitude:";
 }
 
+using namespace Marble;
+
 MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) : QMainWindow(parent), m_sunControlDialog(0)
 {
     setUpdatesEnabled( false );
@@ -55,19 +58,25 @@ MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) : QMainWi
 
     setWindowTitle( tr("Marble - Desktop Globe") );
     setWindowIcon( QIcon(":/icons/marble.png") );
-    setCentralWidget(m_controlView);
+    setCentralWidget( m_controlView );
 
     createActions();
     createMenus();
     createStatusBar();
 
-    readSettings();
     setUpdatesEnabled( true );
 
     m_position = NOT_AVAILABLE;
     m_distance = marbleWidget()->distanceString();
 
+    QTimer::singleShot( 0, this, SLOT( initObject() ) );
+}
+
+void MainWindow::initObject()
+{
+    QCoreApplication::processEvents ();
     setupStatusBar();
+    readSettings();
 }
 
 void MainWindow::createActions()
@@ -125,14 +134,18 @@ void MainWindow::createActions()
      m_showCloudsAct->setStatusTip(tr("Show Real Time Cloud Cover"));
      connect(m_showCloudsAct, SIGNAL(triggered( bool )), this, SLOT( showClouds( bool )));
 
+     m_workOfflineAct = new QAction( QIcon(":/icons/user-offline.png"), tr("&Work Offline"), this);
+     m_workOfflineAct->setCheckable( true );
+     connect(m_workOfflineAct, SIGNAL(triggered( bool )), this, SLOT( workOffline( bool )));
+
      m_showAtmosphereAct = new QAction( tr("&Atmosphere"), this);
      m_showAtmosphereAct->setCheckable( true );
      m_showAtmosphereAct->setStatusTip(tr("Show Atmosphere"));
      connect(m_showAtmosphereAct, SIGNAL(triggered( bool )), this, SLOT( showAtmosphere( bool )));
 
-     m_showSunAct = new QAction( tr("S&un Control"), this);
-     m_showSunAct->setStatusTip(tr("Configure Sun Control"));
-     connect(m_showSunAct, SIGNAL(triggered()), this, SLOT( showSun()));
+     m_controlSunAct = new QAction( tr("S&un Control..."), this);
+     m_controlSunAct->setStatusTip(tr("Configure Sun Control"));
+     connect(m_controlSunAct, SIGNAL(triggered()), this, SLOT( controlSun()));
 
      m_whatsThisAct = new QAction( QIcon(":/icons/help-whatsthis.png"), tr("What's &This"), this);
      m_whatsThisAct->setShortcut(tr("Shift+F1"));
@@ -156,6 +169,7 @@ void MainWindow::createMenus()
     m_fileMenu->addAction(m_exportMapAct);
     m_fileMenu->addAction(m_printAct);
     m_fileMenu->addSeparator();
+    m_fileMenu->addAction(m_workOfflineAct);
     m_fileMenu->addAction(m_quitAct);
 
     m_fileMenu = menuBar()->addMenu(tr("&Edit"));
@@ -175,7 +189,7 @@ void MainWindow::createMenus()
     m_fileMenu->addAction(m_showCloudsAct);
     m_fileMenu->addAction(m_showAtmosphereAct);
     m_fileMenu->addSeparator();
-    m_fileMenu->addAction(m_showSunAct);
+    m_fileMenu->addAction(m_controlSunAct);
 
     m_helpMenu = menuBar()->addMenu(tr("&Help"));
     m_helpMenu->addAction(m_whatsThisAct);
@@ -280,10 +294,10 @@ void MainWindow::showSideBar( bool isChecked )
 
 void MainWindow::copyCoordinates()
 {
-    double lon = m_controlView->marbleWidget()->centerLongitude();
-    double lat = m_controlView->marbleWidget()->centerLatitude();
+    qreal lon = m_controlView->marbleWidget()->centerLongitude();
+    qreal lat = m_controlView->marbleWidget()->centerLatitude();
 
-    QString  positionString = GeoDataPoint( lon, lat, 0.0, GeoDataPoint::Degree ).toString();
+    QString  positionString = GeoDataCoordinates( lon, lat, 0.0, GeoDataCoordinates::Degree ).toString();
     QClipboard  *clipboard = QApplication::clipboard();
 
     clipboard->setText( positionString );
@@ -318,6 +332,16 @@ void MainWindow::showClouds( bool isChecked )
     m_showCloudsAct->setChecked( isChecked ); // Sync state with the GUI
 }
 
+void MainWindow::workOffline( bool offline )
+{
+    if ( offline ) {
+        m_controlView->marbleWidget()->setDownloadManager( 0 );
+    }
+    else {
+        m_controlView->marbleWidget()->setDownloadUrl( "http://download.kde.org/apps/marble/" );
+    }
+}
+
 void MainWindow::showAtmosphere( bool isChecked )
 {
     m_controlView->marbleWidget()->setShowAtmosphere( isChecked );
@@ -325,14 +349,24 @@ void MainWindow::showAtmosphere( bool isChecked )
     m_showAtmosphereAct->setChecked( isChecked ); // Sync state with the GUI
 }
 
-void MainWindow::showSun()
+void MainWindow::controlSun()
 {
-     if (!m_sunControlDialog) m_sunControlDialog = new SunControlWidget( this, m_controlView->sunLocator() );
+    if (!m_sunControlDialog) {
+        m_sunControlDialog = new SunControlWidget( this, m_controlView->sunLocator() );
+        connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
+                 this,               SLOT ( showSun( bool ) ) );
+    }
+
      m_sunControlDialog->show();
      m_sunControlDialog->raise();
      m_sunControlDialog->activateWindow();
 }
 
+void MainWindow::showSun( bool active )
+{
+    m_controlView->marbleWidget()->sunLocator()->setShow( active ); 
+}
+
 void MainWindow::enterWhatsThis()
 {
     QWhatsThis::enterWhatsThisMode();
@@ -456,6 +490,7 @@ void MainWindow::readSettings()
          showSideBar(settings.value("sideBar", true ).toBool());
          showStatusBar(settings.value("statusBar", false ).toBool());
          showClouds(settings.value("showClouds", true ).toBool());
+         workOffline(settings.value("workOffline", false ).toBool());
          showAtmosphere(settings.value("showAtmosphere", true ).toBool());
      settings.endGroup();
 
@@ -490,12 +525,13 @@ void MainWindow::writeSettings()
          settings.setValue( "sideBar", m_sideBarAct->isChecked() );
          settings.setValue( "statusBar", m_statusBarAct->isChecked() );
          settings.setValue( "showClouds", m_showCloudsAct->isChecked() );
+         settings.setValue( "workOffline", m_workOfflineAct->isChecked() );
          settings.setValue( "showAtmosphere", m_showAtmosphereAct->isChecked() );
      settings.endGroup();
 
      settings.beginGroup( "MarbleWidget" );
-         double homeLon = 0;
-         double homeLat = 0;
+         qreal homeLon = 0;
+         qreal homeLat = 0;
          int homeZoom = 0;
          m_controlView->marbleWidget()->home( homeLon, homeLat, homeZoom );
          QString  mapTheme = m_controlView->marbleWidget()->mapThemeId();
