@@ -10,21 +10,26 @@
 
 
 #include "MarbleAbstractFloatItem.h"
-
+#include "MarbleWidget.h"
+#include "MarbleMap.h"
 
 #include <QtGui/QAction>
 #include <QtCore/QDebug>
 #include <QtCore/QRectF>
 #include "GeoPainter.h"
 
+namespace Marble
+{
+
 class MarbleAbstractFloatItemPrivate
 {
   public:
     MarbleAbstractFloatItemPrivate( const QPointF &point, const QSizeF &size )
         : m_position( point ),
           m_size( size ),
-          m_visible( true ), 
-          m_newItemProperties( true )
+          m_visible( true ),
+          m_newItemProperties( true ),
+          m_floatItemMoving( false )
     {
         calculateLayout();
     }
@@ -35,19 +40,19 @@ class MarbleAbstractFloatItemPrivate
 
     void calculateLayout()
     {
-        double marginTop = ( s_marginTop == 0.0 ) ? s_margin : s_marginTop;;
-        double marginBottom = ( s_marginBottom == 0.0 ) ? s_margin : s_marginBottom;;
-        double marginLeft = ( s_marginLeft == 0.0 ) ? s_margin : s_marginLeft;
-        double marginRight = ( s_marginRight == 0.0 ) ? s_margin : s_marginRight;;
+        qreal marginTop = ( s_marginTop == 0.0 ) ? s_margin : s_marginTop;;
+        qreal marginBottom = ( s_marginBottom == 0.0 ) ? s_margin : s_marginBottom;;
+        qreal marginLeft = ( s_marginLeft == 0.0 ) ? s_margin : s_marginLeft;
+        qreal marginRight = ( s_marginRight == 0.0 ) ? s_margin : s_marginRight;;
 
-        m_renderedRect = QRectF( m_position.x() + marginLeft, m_position.y() + marginTop, m_size.width() - ( marginLeft + marginRight ), m_size.height() - ( marginTop + marginBottom ) ); 
-    
-        m_contentRect = QRectF( 
-            m_position.x() + marginLeft + s_padding, 
-            m_position.y() + marginTop + s_padding, 
-            m_size.width() - ( marginLeft + marginRight + 2.0 * s_padding ), 
-            m_size.height() - ( marginTop + marginBottom + 2.0 * s_padding ) 
-        ); 
+        m_renderedRect = QRectF( m_position.x() + marginLeft, m_position.y() + marginTop, m_size.width() - ( marginLeft + marginRight ), m_size.height() - ( marginTop + marginBottom ) );
+
+        m_contentRect = QRectF(
+            m_position.x() + marginLeft + s_padding,
+            m_position.y() + marginTop + s_padding,
+            m_size.width() - ( marginLeft + marginRight + 2.0 * s_padding ),
+            m_size.height() - ( marginTop + marginBottom + 2.0 * s_padding )
+        );
     }
 
     QPointF             m_position;
@@ -59,25 +64,29 @@ class MarbleAbstractFloatItemPrivate
 
     static QPen         s_pen;
     static QBrush       s_background;
-    static double       s_border;
+    static qreal       s_border;
     static QBrush       s_borderBrush;
     static Qt::PenStyle s_borderStyle;
     static QFont        s_font;
-    static double       s_margin;
-    static double       s_marginTop;
-    static double       s_marginBottom;
-    static double       s_marginLeft;
-    static double       s_marginRight;
-    static double       s_padding;
+    static qreal       s_margin;
+    static qreal       s_marginTop;
+    static qreal       s_marginBottom;
+    static qreal       s_marginLeft;
+    static qreal       s_marginRight;
+    static qreal       s_padding;
     static bool         s_pixmapCacheEnabled;
+    static bool         s_positionLocked;
 
     QPixmap             m_cachePixmap;
     bool                m_newItemProperties;
+
+    QPoint              m_floatItemMoveStartPos;
+    bool                m_floatItemMoving;
 };
 
 QPen         MarbleAbstractFloatItemPrivate::s_pen = QPen( Qt::black );
 QBrush       MarbleAbstractFloatItemPrivate::s_background = QBrush( QColor( 192, 192, 192, 192 ) );
-double       MarbleAbstractFloatItemPrivate::s_border = 1.0;
+qreal       MarbleAbstractFloatItemPrivate::s_border = 1.0;
 QBrush       MarbleAbstractFloatItemPrivate::s_borderBrush = QBrush( Qt::black );
 Qt::PenStyle MarbleAbstractFloatItemPrivate::s_borderStyle = Qt::SolidLine;
 #ifdef Q_OS_MACX
@@ -85,17 +94,18 @@ Qt::PenStyle MarbleAbstractFloatItemPrivate::s_borderStyle = Qt::SolidLine;
 #else
     QFont MarbleAbstractFloatItemPrivate::s_font = QFont( "Sans Serif", 8 );
 #endif
-double       MarbleAbstractFloatItemPrivate::s_margin = 0.0;
-double       MarbleAbstractFloatItemPrivate::s_marginTop = 0.0;
-double       MarbleAbstractFloatItemPrivate::s_marginBottom = 0.0;
-double       MarbleAbstractFloatItemPrivate::s_marginLeft = 0.0;
-double       MarbleAbstractFloatItemPrivate::s_marginRight = 0.0;
-double       MarbleAbstractFloatItemPrivate::s_padding = 4.0;
+qreal       MarbleAbstractFloatItemPrivate::s_margin = 0.0;
+qreal       MarbleAbstractFloatItemPrivate::s_marginTop = 0.0;
+qreal       MarbleAbstractFloatItemPrivate::s_marginBottom = 0.0;
+qreal       MarbleAbstractFloatItemPrivate::s_marginLeft = 0.0;
+qreal       MarbleAbstractFloatItemPrivate::s_marginRight = 0.0;
+qreal       MarbleAbstractFloatItemPrivate::s_padding = 4.0;
 bool         MarbleAbstractFloatItemPrivate::s_pixmapCacheEnabled = true;
+bool         MarbleAbstractFloatItemPrivate::s_positionLocked = false;
 
-MarbleAbstractFloatItem::MarbleAbstractFloatItem( const QPointF &point, 
+MarbleAbstractFloatItem::MarbleAbstractFloatItem( const QPointF &point,
                                                   const QSizeF &size )
-    : MarbleAbstractLayer(),
+    : MarbleRenderPlugin(),
       d( new MarbleAbstractFloatItemPrivate( point, size ) )
 {
 }
@@ -119,7 +129,7 @@ QPointF MarbleAbstractFloatItem::position() const
 
 QPointF MarbleAbstractFloatItem::positivePosition( const QRectF& viewPort ) const
 {
-    double x, y;
+    qreal x, y;
     x = ( d->m_position.x() < 0 ) ? viewPort.width() - d->m_size.width() + d->m_position.x() : d->m_position.x();
     y = ( d->m_position.y() < 0 ) ? viewPort.height() - d->m_size.height() + d->m_position.y() : d->m_position.y();
 
@@ -177,7 +187,7 @@ QRectF MarbleAbstractFloatItem::renderedRect() const
 QPainterPath MarbleAbstractFloatItem::backgroundShape() const
 {
     QPainterPath path;
-    path.addRect( QRectF( 0.0, 0.0, d->m_renderedRect.size().width() - 1, d->m_renderedRect.size().height() - 1 ) );
+    path.addRect( QRectF( 0.0, 0.0, d->m_renderedRect.size().width(), d->m_renderedRect.size().height() ) );
     return path;
 }
 
@@ -194,12 +204,12 @@ void MarbleAbstractFloatItem::renderBackground( QPainter* painter )
     painter->restore();
 }
 
-double MarbleAbstractFloatItem::border() const
+qreal MarbleAbstractFloatItem::border() const
 {
     return d->s_border;
 }
 
-void MarbleAbstractFloatItem::setBorder( double border )
+void MarbleAbstractFloatItem::setBorder( qreal border )
 {
     d->s_border = border;
     d->m_newItemProperties = true;
@@ -219,7 +229,7 @@ void MarbleAbstractFloatItem::setBorderBrush( const QBrush &borderBrush )
 Qt::PenStyle MarbleAbstractFloatItem::borderStyle () const
 {
     return d->s_borderStyle;
-} 
+}
 
 void MarbleAbstractFloatItem::setBorderStyle( Qt::PenStyle borderStyle )
 {
@@ -238,12 +248,12 @@ void MarbleAbstractFloatItem::setFont( const QFont &font )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::margin() const
+qreal MarbleAbstractFloatItem::margin() const
 {
     return d->s_margin;
-} 
+}
 
-void MarbleAbstractFloatItem::setMargin( double margin )
+void MarbleAbstractFloatItem::setMargin( qreal margin )
 {
     d->s_margin = margin;
 
@@ -251,12 +261,12 @@ void MarbleAbstractFloatItem::setMargin( double margin )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::marginTop() const
+qreal MarbleAbstractFloatItem::marginTop() const
 {
     return d->s_marginTop;
-} 
+}
 
-void MarbleAbstractFloatItem::setMarginTop( double marginTop )
+void MarbleAbstractFloatItem::setMarginTop( qreal marginTop )
 {
     d->s_marginTop = marginTop;
 
@@ -264,12 +274,12 @@ void MarbleAbstractFloatItem::setMarginTop( double marginTop )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::marginBottom() const
+qreal MarbleAbstractFloatItem::marginBottom() const
 {
     return d->s_marginBottom;
-} 
+}
 
-void MarbleAbstractFloatItem::setMarginBottom( double marginBottom )
+void MarbleAbstractFloatItem::setMarginBottom( qreal marginBottom )
 {
     d->s_marginBottom = marginBottom;
 
@@ -277,12 +287,12 @@ void MarbleAbstractFloatItem::setMarginBottom( double marginBottom )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::marginLeft() const
+qreal MarbleAbstractFloatItem::marginLeft() const
 {
     return d->s_marginLeft;
-} 
+}
 
-void MarbleAbstractFloatItem::setMarginLeft( double marginLeft )
+void MarbleAbstractFloatItem::setMarginLeft( qreal marginLeft )
 {
     d->s_marginLeft = marginLeft;
 
@@ -290,12 +300,12 @@ void MarbleAbstractFloatItem::setMarginLeft( double marginLeft )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::marginRight() const
+qreal MarbleAbstractFloatItem::marginRight() const
 {
     return d->s_marginRight;
-} 
+}
 
-void MarbleAbstractFloatItem::setMarginRight( double marginRight )
+void MarbleAbstractFloatItem::setMarginRight( qreal marginRight )
 {
     d->s_marginRight = marginRight;
 
@@ -303,12 +313,12 @@ void MarbleAbstractFloatItem::setMarginRight( double marginRight )
     d->m_newItemProperties = true;
 }
 
-double MarbleAbstractFloatItem::padding () const
+qreal MarbleAbstractFloatItem::padding () const
 {
     return d->s_padding;
 }
 
-void MarbleAbstractFloatItem::setPadding( double padding )
+void MarbleAbstractFloatItem::setPadding( qreal padding )
 {
     d->s_padding = padding;
 
@@ -316,6 +326,16 @@ void MarbleAbstractFloatItem::setPadding( double padding )
     d->m_newItemProperties = true;
 }
 
+bool MarbleAbstractFloatItem::positionLocked() const
+{
+       return d->s_positionLocked;
+}
+
+void MarbleAbstractFloatItem::setPositionLocked( bool enabled )
+{
+       d->s_positionLocked = enabled;
+}
+
 bool MarbleAbstractFloatItem::needsUpdate( ViewportParams *viewport )
 {
     Q_UNUSED( viewport );
@@ -334,7 +354,7 @@ void MarbleAbstractFloatItem::setPixmapCacheEnabled( bool pixmapCacheEnabled )
     d->m_newItemProperties = true;
 }
 
-QString MarbleAbstractFloatItem::renderPolicy() const 
+QString MarbleAbstractFloatItem::renderPolicy() const
 {
     return "ALWAYS";
 }
@@ -355,13 +375,12 @@ bool MarbleAbstractFloatItem::render( GeoPainter *painter, ViewportParams *viewp
             painter->drawPixmap( positivePosition( painter->viewport() ), d->m_cachePixmap );
             return true;
         }
-    
-        // Reinitialize cachePixmap if the float item changes its size 
-        // or other important common properties 
+
+            // Reinitialize cachePixmap if the float item changes its size
+        // or other important common properties
         if ( ( d->s_pixmapCacheEnabled && d->m_newItemProperties ) || d->m_cachePixmap.isNull() ) {
             // Add extra space for the border
-            QSize cachePixmapSize = d->m_size.toSize();
-
+            QSize cachePixmapSize = d->m_size.toSize() + QSize( 1, 1 ); // adding a pixel for rounding errors
 
             if ( d->m_size.isValid() && !d->m_size.isNull() ) {
                 d->m_cachePixmap = QPixmap( cachePixmapSize ).copy();
@@ -437,5 +456,93 @@ bool MarbleAbstractFloatItem::renderOnMap( GeoPainter     *painter,
     return true;
 }
 
-#include "MarbleAbstractFloatItem.moc"
 
+bool MarbleAbstractFloatItem::eventFilter( QObject *object, QEvent *e )
+{
+    if ( !enabled() || !visible() || positionLocked() ) {
+        return false;
+    }
+
+    MarbleWidget *widget = dynamic_cast<MarbleWidget*>(object);
+    if ( !widget ) {
+        return false;
+    }
+
+    // Move float items
+    bool cursorAboveFloatItem(false);
+    if ( e->type() == QEvent::MouseMove
+                || e->type() == QEvent::MouseButtonPress
+                || e->type() == QEvent::MouseButtonRelease )
+    {
+        QMouseEvent *event = static_cast<QMouseEvent*>(e);
+        QRectF floatItemRect = QRectF(positivePosition(QRectF(0,0,widget->width(),
+                widget->height())), size() + QSize(1,1));
+
+        // Click and move above a float item triggers moving the float item
+        if ( floatItemRect.contains(event->pos()) ) {
+            cursorAboveFloatItem = true;
+
+            if ( e->type() == QEvent::MouseButtonPress && event->button() == Qt::LeftButton ) {
+                d->m_floatItemMoveStartPos = event->pos();
+                d->m_floatItemMoving = true;
+                return true;
+            }
+        }
+
+        if ( e->type() == QEvent::MouseMove && event->buttons() & Qt::LeftButton
+            && ( cursorAboveFloatItem || d->m_floatItemMoving ) )
+        {
+            d->m_floatItemMoving = true;
+            const QPoint &point = event->pos();
+            QPointF position = floatItemRect.topLeft();
+            qreal newX = position.x()+point.x()-d->m_floatItemMoveStartPos.x();
+            qreal newY = position.y()+point.y()-d->m_floatItemMoveStartPos.y();
+            if ( newX >= 0 && newY >= 0 ) {
+                // Docking behavior
+                const qreal dockArea = 60.0; // Alignment area width/height
+                const qreal dockJump = 30.0; // Alignment indicator jump size
+                if ( widget->width()-size().width()-newX < dockArea ) {
+                    newX = qMin(qreal(-1.0), size().width()+newX-widget->width());
+                    if (d->m_floatItemMoveStartPos.x()<event->pos().x()) {
+                        // Indicate change to right alignment with a short jump
+                        newX = qMax( newX, -(dockArea-dockJump) );
+                    }
+                }
+                if ( widget->height()-size().height()-newY < dockArea ) {
+                    newY = qMin(qreal(-1.0),size().height()+newY-widget->height());
+                    if (d->m_floatItemMoveStartPos.y()<event->pos().y()) {
+                       // Indicate change to bottom alignment with a short jump
+                       newY = qMax( newY, -(dockArea-dockJump) );
+                    }
+                }
+
+                setPosition(QPointF(newX,newY));
+                QRect newFloatItemRect = QRectF(positivePosition(QRect(0,0,widget->width(),widget->height())), size() ).toRect();
+                d->m_floatItemMoveStartPos = event->pos();
+                QRegion dirtyRegion(floatItemRect.toRect());
+                dirtyRegion = dirtyRegion.united(newFloatItemRect);
+
+                widget->setAttribute( Qt::WA_NoSystemBackground,  false );
+                widget->repaint(dirtyRegion);
+                widget->setAttribute( Qt::WA_NoSystemBackground,  widget->map()->mapCoversViewport() );
+                return true;
+            }
+        }
+
+        if ( e->type() == QEvent::MouseButtonRelease ) {
+            d->m_floatItemMoving = false;
+        }
+
+        // Adjusting Cursor shape
+        if ( cursorAboveFloatItem || d->m_floatItemMoving ) {
+            widget->setCursor(QCursor(Qt::SizeAllCursor));
+            return true;
+        }
+    }
+
+    return false;
+}
+
+}
+
+#include "MarbleAbstractFloatItem.moc"
