@@ -5,7 +5,7 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
+// Copyright 2006-2009 Torsten Rahn <tackat@kde.org>"
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>"
 // Copyright 2008      Carlos Licea <carlos.licea@kdemail.net>
 //
@@ -20,6 +20,7 @@
 
 // Qt
 #include <QtCore/QCoreApplication>
+#include <QtCore/QDebug>
 #include <QtCore/QLocale>
 #include <QtCore/QTranslator>
 #include <QtCore/QAbstractItemModel>
@@ -32,9 +33,6 @@
 //#include <QtDBus/QDBusConnection>
 
 // Marble
-#include "CrossHairFloatItem.h"
-#include "MapScaleFloatItem.h"
-
 #include "AbstractProjection.h"
 #include "AbstractScanlineTextureMapper.h"
 #include "BoundingBox.h"
@@ -42,13 +40,14 @@
 #include "FileViewModel.h"
 #include "FileStoragePolicy.h"
 #include "GeoDataFeature.h"
-#include "GeoDataPoint.h"
+#include "GeoDataCoordinates.h"
 #include "GeoSceneDocument.h"
 #include "GeoSceneHead.h"
 #include "GeoSceneZoom.h"
 #include "GpxFileViewItem.h"
 #include "HttpDownloadManager.h"
 #include "MarbleDirs.h"
+#include "MarbleLocale.h"
 #include "MarbleModel.h"
 #include "MarblePlacemarkModel.h"
 #include "MeasureTool.h"
@@ -59,8 +58,11 @@
 #include "ViewParams.h"
 #include "ViewportParams.h"
 
+#include "MarbleRenderPlugin.h"
+
 #include "gps/GpsLayer.h"
 
+using namespace Marble;
 
 #ifdef Q_CC_MSVC
 # ifndef KDEWIN_MATH_H
@@ -77,6 +79,280 @@ MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent )
     /* NOOP */
 }
 
+void MarbleMapPrivate::construct()
+{
+    // Some point that tackat defined. :-)
+    m_parent->setHome( -9.4, 54.8, 1050 );
+
+    m_parent->connect( m_model, SIGNAL( themeChanged( QString ) ),
+                       m_parent, SIGNAL( themeChanged( QString ) ) );
+    m_parent->connect( m_model, SIGNAL( modelChanged() ),
+                       m_parent, SLOT( updateChangedMap() ) );
+
+    m_parent->connect( m_model, SIGNAL( regionChanged( BoundingBox& ) ) ,
+                       m_parent, SLOT( updateRegion( BoundingBox& ) ) );
+
+    m_justModified = false;
+
+    m_measureTool = new MeasureTool( m_model, m_parent );
+
+    m_parent->connect( m_model, SIGNAL( timeout() ),
+                       m_parent, SLOT( updateGps() ) );
+
+
+    m_logzoom  = 0;
+    m_zoomStep = 40;
+
+    m_parent->goHome();
+
+    // FloatItems
+    m_showFrameRate = false;
+
+
+    m_parent->connect( m_model->sunLocator(), SIGNAL( updateSun() ),
+                        m_parent,              SLOT( updateSun() ) );
+    m_parent->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
+                        m_parent,              SLOT( centerSun() ) );
+}
+
+// Used to be resizeEvent()
+void MarbleMapPrivate::doResize()
+{
+    QSize size(m_parent->width(), m_parent->height());
+    m_viewParams.viewport()->setSize( size );
+    // Recreate the canvas image with the new size.
+    m_viewParams.setCanvasImage( new QImage( m_parent->width(), m_parent->height(),
+                                             QImage::Format_ARGB32_Premultiplied ));
+
+    if ( m_viewParams.showAtmosphere() ) {
+        drawAtmosphere();
+    }
+
+    // Recreate the 
+    m_viewParams.setCoastImage( new QImage( m_parent->width(), m_parent->height(),
+                                            QImage::Format_ARGB32_Premultiplied ));
+
+    m_justModified = true;
+}
+
+void  MarbleMapPrivate::paintMarbleSplash( GeoPainter &painter, QRect &dirtyRect )
+{
+    painter.save();
+
+    QPixmap logoPixmap( MarbleDirs::path( "svg/marble-logo-inverted-72dpi.png" ) );
+
+    if ( logoPixmap.width() > m_parent->width() * 0.7 || logoPixmap.height() > m_parent->height() * 0.7 )
+    {
+        logoPixmap = logoPixmap.scaled( QSize( m_parent->width(), m_parent->height() ) * 0.7, Qt::KeepAspectRatio, Qt::SmoothTransformation );
+    }
+
+    QPoint logoPosition( ( m_parent->width()  - logoPixmap.width() ) / 2 , 
+                            ( m_parent->height() - logoPixmap.height() ) / 2 ); 
+    painter.drawPixmap( logoPosition, logoPixmap );
+
+    QString message = ""; // "Please assign a map theme!";
+
+    painter.setPen( Qt::white );
+
+    int yTop = logoPosition.y() + logoPixmap.height() + 10;
+    QRect textRect( 0, yTop,
+                    m_parent->width(), m_parent->height() - yTop );
+    painter.drawText( textRect, Qt::AlignHCenter | Qt::AlignTop, message ); 
+
+    painter.restore();
+}
+
+void MarbleMapPrivate::drawAtmosphere()
+{
+    // Only draw an atmosphere if planet is earth
+    GeoSceneDocument * mapTheme = m_viewParams.mapTheme();
+    if ( mapTheme ) {
+        if ( mapTheme->head()->target() != "earth" )
+            return;
+    }
+
+    // Only draw an atmosphere if projection is spherical
+    if ( m_viewParams.projection() != Spherical )
+        return;
+
+    // No use to draw atmosphere if it's not visible in the area. 
+    if ( m_viewParams.viewport()->mapCoversViewport() )
+        return;
+
+    // Ok, now we know that at least a little of the atmosphere is
+    // visible, if nothing else in the corners.  Draw the atmosphere
+    // by using a circular gradient.  This is a pure visual effect and
+    // has nothing to do with real physics.
+
+    int  imageHalfWidth  = m_parent->width() / 2;
+    int  imageHalfHeight = m_parent->height() / 2;
+
+    // Recalculate the atmosphere effect and paint it to canvasImage.
+    QRadialGradient grad1( QPointF( imageHalfWidth, imageHalfHeight ),
+                           1.05 * m_parent->radius() );
+    grad1.setColorAt( 0.91, QColor( 255, 255, 255, 255 ) );
+    grad1.setColorAt( 1.00, QColor( 255, 255, 255, 0 ) );
+
+    QBrush    brush1( grad1 );
+    QPen      pen1( Qt::NoPen );
+    QPainter  painter( m_viewParams.canvasImage() );
+    painter.setBrush( brush1 );
+    painter.setPen( pen1 );
+    painter.setRenderHint( QPainter::Antialiasing, false );
+    painter.drawEllipse( imageHalfWidth  - (int)( (qreal)(m_parent->radius()) * 1.05 ),
+                         imageHalfHeight - (int)( (qreal)(m_parent->radius()) * 1.05 ),
+                         (int)( 2.1 * (qreal)(m_parent->radius()) ),
+                         (int)( 2.1 * (qreal)(m_parent->radius()) ) );
+}
+
+void MarbleMapPrivate::drawFog( QPainter &painter )
+{
+    if ( m_viewParams.projection() != Spherical)
+        return;
+
+    // No use to draw the fog if it's not visible in the area. 
+    if ( m_viewParams.viewport()->mapCoversViewport() )
+        return;
+
+    int  imgWidth2  = m_parent->width() / 2;
+    int  imgHeight2 = m_parent->height() / 2;
+
+    // Recalculate the atmosphere effect and paint it to canvasImage.
+    QRadialGradient grad1( QPointF( imgWidth2, imgHeight2 ), m_parent->radius() );
+
+    // FIXME: Add a cosine relationship
+    grad1.setColorAt( 0.85, QColor( 255, 255, 255, 0 ) );
+    grad1.setColorAt( 1.00, QColor( 255, 255, 255, 64 ) );
+
+    QBrush    brush1( grad1 );
+    QPen      pen1( Qt::NoPen );
+
+    painter.save();
+
+    painter.setBrush( brush1 );
+    painter.setPen( pen1 );
+    painter.setRenderHint( QPainter::Antialiasing, false );
+
+    // FIXME: Cut out what's really needed
+    painter.drawEllipse( imgWidth2  - m_parent->radius(),
+                         imgHeight2 - m_parent->radius(),
+                         2 * m_parent->radius(),
+                         2 * m_parent->radius() );
+
+    painter.restore();
+}
+
+void MarbleMapPrivate::setBoundingBox()
+{
+    QVector<QPointF>  points;
+    Quaternion        temp;
+
+    if ( m_parent->globalQuaternion( 0, 0, temp) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+    if ( m_parent->globalQuaternion( m_parent->width() / 2, 0, temp ) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+
+    if ( m_parent->globalQuaternion( m_parent->width(), 0, temp ) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+    if ( m_parent->globalQuaternion( 0, m_parent->height(), temp ) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+
+    if ( m_parent->globalQuaternion( m_parent->width()/2, m_parent->height(), temp ) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+
+    if ( m_parent->globalQuaternion( m_parent->width(), m_parent->height(), temp ) ) {
+        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
+    }
+
+    m_viewParams.viewport()->setBoundingBox( BoundingBox( points ) );
+}
+
+void MarbleMapPrivate::paintGround( GeoPainter &painter, QRect &dirtyRect )
+{
+    if ( !m_viewParams.mapTheme() ) 
+    {
+        qDebug() << "No theme yet!";
+        paintMarbleSplash( painter, dirtyRect );
+        return;
+    }
+
+    bool  doClip = false;
+    if ( m_viewParams.projection() == Spherical )
+        doClip = ( m_viewParams.radius() > m_viewParams.canvasImage()->width() / 2
+                   || m_viewParams.radius() > m_viewParams.canvasImage()->height() / 2 );
+
+    m_model->paintGlobe( &painter,
+                            m_parent->width(), m_parent->height(), &m_viewParams,
+                            m_parent->needsUpdate()
+                            || m_viewParams.canvasImage()->isNull(),
+                            dirtyRect );
+    // FIXME: this is ugly, add method updatePlanetAxis() to ViewParams
+    m_viewParams.setPlanetAxisUpdated( m_viewParams.planetAxis() );
+    // FIXME: this is ugly, add method updateRadius() to ViewParams
+    m_viewParams.setRadiusUpdated( m_viewParams.radius() );
+    m_justModified                   = false;
+
+    // FIXME: This is really slow. That's why we defer this to
+    //        PrintQuality. Either cache on a pixmap - or maybe
+    //        better: Add to GlobeScanlineTextureMapper.
+
+    if ( m_viewParams.mapQuality() == Marble::Print )
+        drawFog(painter);
+}
+
+void MarbleMapPrivate::paintOverlay( GeoPainter &painter, QRect &dirtyRect)
+{
+    if ( !m_viewParams.mapTheme() ) 
+    {
+        return;
+    }
+
+    // FIXME: Add this stuff into the Layermanager as something to be 
+    // called before the float items.
+
+    bool antialiased = false;
+
+    if (   m_viewParams.mapQuality() == Marble::High
+        || m_viewParams.mapQuality() == Marble::Print ) {
+            antialiased = true;
+    }
+
+    m_measureTool->paint( &painter, m_viewParams.viewport(), antialiased );
+
+    // FIXME: Get rid of this:
+    // Set the Bounding Box 
+    setBoundingBox();
+}
+
+void MarbleMapPrivate::paintFps( GeoPainter &painter, QRect &dirtyRect, qreal fps)
+{
+    Q_UNUSED(dirtyRect);
+
+    if ( m_showFrameRate == true ) {
+        QString fpsString = QString( "Speed: %1 fps" ).arg( fps, 5, 'f', 1, QChar(' ') );
+
+        QPoint fpsLabelPos( 10, 20 );
+
+        painter.setFont( QFont( "Sans Serif", 10 ) );
+
+        painter.setPen( Qt::black );
+        painter.setBrush( Qt::black );
+        painter.drawText( fpsLabelPos, fpsString );
+
+        painter.setPen( Qt::white );
+        painter.setBrush( Qt::white );
+        painter.drawText( fpsLabelPos.x() - 1, fpsLabelPos.y() - 1, fpsString );
+    }
+}
+
+
+// ----------------------------------------------------------------
+
 
 MarbleMap::MarbleMap()
     : d( new MarbleMapPrivate( this ) )
@@ -114,51 +390,6 @@ MarbleMap::~MarbleMap()
     delete d;
 }
 
-void MarbleMapPrivate::construct()
-{
-    // Some point that tackat defined. :-)
-    m_parent->setHome( -9.4, 54.8, 1050 );
-
-    m_parent->connect( m_model, SIGNAL( themeChanged( QString ) ),
-                       m_parent, SIGNAL( themeChanged( QString ) ) );
-    m_parent->connect( m_model, SIGNAL( modelChanged() ),
-                       m_parent, SLOT( updateChangedMap() ) );
-
-    m_parent->connect( m_model, SIGNAL( regionChanged( BoundingBox& ) ) ,
-                       m_parent, SLOT( updateRegion( BoundingBox& ) ) );
-
-    m_justModified = false;
-
-    m_measureTool = new MeasureTool( m_parent );
-
-    m_parent->connect( m_model, SIGNAL( timeout() ),
-                       m_parent, SLOT( updateGps() ) );
-
-
-    m_logzoom  = 0;
-    m_zoomStep = 40;
-
-    m_parent->goHome();
-
-    // FloatItems
-    m_showFrameRate = false;
-
-    // Map translation
-    QString      locale = QLocale::system().name();
-    QTranslator  translator;
-    translator.load(QString("marblemap_") + locale);
-    QCoreApplication::installTranslator(&translator);
-
-    m_parent->connect( m_model->sunLocator(), SIGNAL( updateSun() ),
-                        m_parent,              SLOT( updateSun() ) );
-    m_parent->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
-                        m_parent,              SLOT( centerSun() ) );
-}
-
-
-// ----------------------------------------------------------------
-
-
 MarbleModel *MarbleMap::model() const
 {
     return d->m_model;
@@ -224,7 +455,7 @@ int MarbleMap::radius() const
     return d->m_viewParams.radius();
 }
 
-void MarbleMap::setRadius(const int radius)
+void MarbleMap::setRadius(int radius)
 {
     d->m_viewParams.setRadius( radius );
 
@@ -257,13 +488,13 @@ QItemSelectionModel *MarbleMap::placeMarkSelectionModel() const
     return d->m_model->placeMarkSelectionModel();
 }
 
-double MarbleMap::moveStep()
+qreal MarbleMap::moveStep()
 {
-    if ( radius() < sqrt( (double)(width() * width() + height() * height()) ) )
+    if ( radius() < sqrt( (qreal)(width() * width() + height() * height()) ) )
 	return 180.0 * 0.1;
     else
-	return 180.0 * atan( (double)width()
-		     / (double)( 2 * radius() ) ) * 0.2;
+	return 180.0 * atan( (qreal)width()
+		     / (qreal)( 2 * radius() ) ) * 0.2;
 }
 
 int MarbleMap::zoom() const
@@ -272,7 +503,7 @@ int MarbleMap::zoom() const
 }
 
 
-double MarbleMap::distance() const
+qreal MarbleMap::distance() const
 {
     // Due to Marble's orthographic projection ("we have no focus")
     // it's actually not possible to calculate a "real" distance.
@@ -284,37 +515,37 @@ double MarbleMap::distance() const
     // reality. Therefore we assume that the average window width
     // (about 800 pixels) equals the viewing angle of a human being.
 
-    const double VIEW_ANGLE = 110.0;
+    const qreal VIEW_ANGLE = 110.0;
 
-    return ( EARTH_RADIUS * 0.4
-            / (double)( radius() )
+    return ( model()->planetRadius() * 0.4
+            / (qreal)( radius() )
             / tan( 0.5 * VIEW_ANGLE * DEG2RAD ) );
 }
 
-void MarbleMap::setDistance( double distance )
+void MarbleMap::setDistance( qreal distance )
 {
-    const double VIEW_ANGLE = 110.0;
+    const qreal VIEW_ANGLE = 110.0;
 
-    setRadius( (int)( EARTH_RADIUS * 0.4
+    setRadius( (int)( model()->planetRadius() * 0.4
             / distance
             / tan( 0.5 * VIEW_ANGLE * DEG2RAD ) ) );
 }
 
-double MarbleMap::centerLatitude() const
+qreal MarbleMap::centerLatitude() const
 {
     // Calculate translation of center point
-    double  centerLon;
-    double  centerLat;
+    qreal  centerLon;
+    qreal  centerLat;
 
     d->m_viewParams.centerCoordinates( centerLon, centerLat );
     return centerLat * RAD2DEG;
 }
 
-double MarbleMap::centerLongitude() const
+qreal MarbleMap::centerLongitude() const
 {
     // Calculate translation of center point
-    double  centerLon;
-    double  centerLat;
+    qreal  centerLon;
+    qreal  centerLat;
 
     d->m_viewParams.centerCoordinates(centerLon, centerLat);
     return centerLon * RAD2DEG;
@@ -341,9 +572,14 @@ void MarbleMap::addPlaceMarkFile( const QString &filename )
     d->m_model->addPlaceMarkFile( filename );
 }
 
-void MarbleMap::addPlaceMarkData( const QString &data )
+void MarbleMap::addPlaceMarkData( const QString &data, const QString &key )
 {
-    d->m_model->addPlaceMarkData( data );
+    d->m_model->addPlaceMarkData( data, key );
+}
+
+void MarbleMap::removePlaceMarkKey( const QString &key )
+{
+    d->m_model->removePlaceMarkKey( key );
 }
 
 
@@ -393,6 +629,21 @@ bool MarbleMap::showAtmosphere() const
     return d->m_viewParams.showAtmosphere();
 }
 
+bool MarbleMap::showCrosshairs() const
+{
+    bool visible = false;
+
+    QList<MarbleRenderPlugin *> pluginList = renderPlugins();
+    QList<MarbleRenderPlugin *>::const_iterator i;
+    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+        if ( (*i)->nameId() == "crosshairs" ) {
+            visible = (*i)->visible();
+        }
+    }
+
+    return visible;
+}
+
 bool MarbleMap::showPlaces() const
 {
     return propertyValue( "places" );
@@ -509,7 +760,7 @@ void MarbleMap::rotateBy(const Quaternion& incRot)
     d->m_viewParams.setPlanetAxis( incRot * d->m_viewParams.planetAxis() );
 }
 
-void MarbleMap::rotateBy( const double& deltaLon, const double& deltaLat)
+void MarbleMap::rotateBy( const qreal& deltaLon, const qreal& deltaLat)
 {
     Quaternion  rotPhi( 1.0, deltaLat / 180.0, 0.0, 0.0 );
     Quaternion  rotTheta( 1.0, 0.0, deltaLon / 180.0, 0.0 );
@@ -522,7 +773,7 @@ void MarbleMap::rotateBy( const double& deltaLon, const double& deltaLat)
 }
 
 
-void MarbleMap::centerOn(const double& lon, const double& lat)
+void MarbleMap::centerOn(const qreal& lon, const qreal& lat)
 {
     Quaternion  quat;
     quat.createFromEuler( -lat * DEG2RAD, lon * DEG2RAD, 0.0 );
@@ -537,28 +788,29 @@ void MarbleMap::centerOn(const QModelIndex& index)
     selectionModel->clear();
 
     if ( index.isValid() ) {
-        const GeoDataPoint point = index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataPoint>();
+        const GeoDataCoordinates point = index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
   
-        double  lon;
-        double  lat;
+        qreal  lon;
+        qreal  lat;
         point.geoCoordinates( lon, lat );
 
         centerOn( lon * RAD2DEG, lat * RAD2DEG );
 
         selectionModel->select( index, QItemSelectionModel::SelectCurrent );
-        d->m_crosshair.setEnabled( true );
+//        Maybe add some similar functionality later on again:
+//        d->m_crosshair.setEnabled( true );
     }
-    else
-        d->m_crosshair.setEnabled( false );
+//    else
+//        d->m_crosshair.setEnabled( false );
 }
 
 
-void MarbleMap::setCenterLatitude( double lat )
+void MarbleMap::setCenterLatitude( qreal lat )
 {
     centerOn( centerLongitude(), lat );
 }
 
-void MarbleMap::setCenterLongitude( double lon )
+void MarbleMap::setCenterLongitude( qreal lon )
 {
     centerOn( lon, centerLatitude() );
 }
@@ -584,19 +836,19 @@ void MarbleMap::setProjection( Projection projection )
     setNeedsUpdate();
 }
 
-void MarbleMap::home( double &lon, double &lat, int& zoom )
+void MarbleMap::home( qreal &lon, qreal &lat, int& zoom )
 {
-    d->m_homePoint.geoCoordinates( lon, lat, GeoDataPoint::Degree );
+    d->m_homePoint.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
     zoom = d->m_homeZoom;
 }
 
-void MarbleMap::setHome( const double lon, const double lat, const int zoom)
+void MarbleMap::setHome( qreal lon, qreal lat, int zoom)
 {
-    d->m_homePoint = GeoDataPoint( lon, lat, 0, GeoDataPoint::Degree );
+    d->m_homePoint = GeoDataCoordinates( lon, lat, 0, GeoDataCoordinates::Degree );
     d->m_homeZoom = zoom;
 }
 
-void MarbleMap::setHome(const GeoDataPoint& homePoint, int zoom)
+void MarbleMap::setHome(const GeoDataCoordinates& homePoint, int zoom)
 {
     d->m_homePoint = homePoint;
     d->m_homeZoom = zoom;
@@ -634,36 +886,16 @@ void MarbleMap::moveDown()
     rotateBy( 0, +moveStep() );
 }
 
-// Used to be resizeEvent()
-void MarbleMapPrivate::doResize()
-{
-    QSize size(m_parent->width(), m_parent->height());
-    m_viewParams.viewport()->setSize( size );
-    // Recreate the canvas image with the new size.
-    m_viewParams.setCanvasImage( new QImage( m_parent->width(), m_parent->height(),
-                                             QImage::Format_ARGB32_Premultiplied ));
-
-    if ( m_viewParams.showAtmosphere() ) {
-        drawAtmosphere();
-    }
-
-    // Recreate the 
-    m_viewParams.setCoastImage( new QImage( m_parent->width(), m_parent->height(),
-                                            QImage::Format_ARGB32_Premultiplied ));
-
-    m_justModified = true;
-}
-
 int MarbleMap::northPoleY()
 {
-    Quaternion  northPole     = GeoDataPoint( 0.0, M_PI * 0.5 ).quaternion();
+    Quaternion  northPole     = GeoDataCoordinates( 0.0, M_PI * 0.5 ).quaternion();
     Quaternion  invPlanetAxis = d->m_viewParams.planetAxis().inverse();
 
     northPole.rotateAroundAxis( invPlanetAxis );
     return (int)( d->m_viewParams.radius() * northPole.v[Q_Y] );
 }
 
-bool MarbleMap::screenCoordinates( const double lon, const double lat,
+bool MarbleMap::screenCoordinates( qreal lon, qreal lat,
                                    int& x, int& y )
 {
     return d->m_viewParams.currentProjection()
@@ -672,9 +904,9 @@ bool MarbleMap::screenCoordinates( const double lon, const double lat,
                              x, y );
 }
 
-bool MarbleMap::geoCoordinates( const int x, const int y,
-                                double& lon, double& lat,
-                                GeoDataPoint::Unit unit )
+bool MarbleMap::geoCoordinates( int x, int y,
+                                qreal& lon, qreal& lat,
+                                GeoDataCoordinates::Unit unit )
 {
     return d->m_viewParams.currentProjection()
         ->geoCoordinates( x, y, d->m_viewParams.viewport(),
@@ -686,17 +918,17 @@ bool MarbleMap::globalQuaternion( int x, int y, Quaternion &q)
     int  imageHalfWidth  = width() / 2;
     int  imageHalfHeight = height() / 2;
 
-    const double  inverseRadius = 1.0 / (double)(radius());
+    const qreal  inverseRadius = 1.0 / (qreal)(radius());
 
-    if ( radius() > sqrt( (double)(( x - imageHalfWidth ) * ( x - imageHalfWidth )
+    if ( radius() > sqrt( (qreal)(( x - imageHalfWidth ) * ( x - imageHalfWidth )
         + ( y - imageHalfHeight ) * ( y - imageHalfHeight )) ) )
     {
-        double qx = inverseRadius * (double)( x - imageHalfWidth );
-        double qy = inverseRadius * (double)( y - imageHalfHeight );
-        double qr = 1.0 - qy * qy;
+        qreal qx = inverseRadius * (qreal)( x - imageHalfWidth );
+        qreal qy = inverseRadius * (qreal)( y - imageHalfHeight );
+        qreal qr = 1.0 - qy * qy;
 
-        double qr2z = qr - qx * qx;
-        double qz = ( qr2z > 0.0 ) ? sqrt( qr2z ) : 0.0;
+        qreal qr2z = qr - qx * qx;
+        qreal qz = ( qr2z > 0.0 ) ? sqrt( qr2z ) : 0.0;
 
         Quaternion  qpos( 0.0, qx, qy, qz );
         qpos.rotateAroundAxis( planetAxis() );
@@ -708,112 +940,6 @@ bool MarbleMap::globalQuaternion( int x, int y, Quaternion &q)
     }
 }
 
-
-void MarbleMapPrivate::drawAtmosphere()
-{
-    // Only draw an atmosphere if projection is spherical
-    if ( m_viewParams.projection() != Spherical )
-        return;
-
-    // No use to draw atmosphere if it's not visible in the area. 
-    if ( m_viewParams.viewport()->mapCoversViewport() )
-        return;
-
-    // Ok, now we know that at least a little of the atmosphere is
-    // visible, if nothing else in the corners.  Draw the atmosphere
-    // by using a circular gradient.  This is a pure visual effect and
-    // has nothing to do with real physics.
-
-    int  imageHalfWidth  = m_parent->width() / 2;
-    int  imageHalfHeight = m_parent->height() / 2;
-
-    // Recalculate the atmosphere effect and paint it to canvasImage.
-    QRadialGradient grad1( QPointF( imageHalfWidth, imageHalfHeight ),
-                           1.05 * m_parent->radius() );
-    grad1.setColorAt( 0.91, QColor( 255, 255, 255, 255 ) );
-    grad1.setColorAt( 1.00, QColor( 255, 255, 255, 0 ) );
-
-    QBrush    brush1( grad1 );
-    QPen      pen1( Qt::NoPen );
-    QPainter  painter( m_viewParams.canvasImage() );
-    painter.setBrush( brush1 );
-    painter.setPen( pen1 );
-    painter.setRenderHint( QPainter::Antialiasing, false );
-    painter.drawEllipse( imageHalfWidth  - (int)( (double)(m_parent->radius()) * 1.05 ),
-                         imageHalfHeight - (int)( (double)(m_parent->radius()) * 1.05 ),
-                         (int)( 2.1 * (double)(m_parent->radius()) ),
-                         (int)( 2.1 * (double)(m_parent->radius()) ) );
-}
-
-
-void MarbleMapPrivate::drawFog( QPainter &painter )
-{
-    if ( m_viewParams.projection() != Spherical)
-        return;
-
-    // No use to draw the fog if it's not visible in the area. 
-    if ( m_viewParams.viewport()->mapCoversViewport() )
-        return;
-
-    int  imgWidth2  = m_parent->width() / 2;
-    int  imgHeight2 = m_parent->height() / 2;
-
-    // Recalculate the atmosphere effect and paint it to canvasImage.
-    QRadialGradient grad1( QPointF( imgWidth2, imgHeight2 ), m_parent->radius() );
-
-    // FIXME: Add a cosine relationship
-    grad1.setColorAt( 0.85, QColor( 255, 255, 255, 0 ) );
-    grad1.setColorAt( 1.00, QColor( 255, 255, 255, 64 ) );
-
-    QBrush    brush1( grad1 );
-    QPen      pen1( Qt::NoPen );
-
-    painter.save();
-
-    painter.setBrush( brush1 );
-    painter.setPen( pen1 );
-    painter.setRenderHint( QPainter::Antialiasing, false );
-
-    // FIXME: Cut out what's really needed
-    painter.drawEllipse( imgWidth2  - m_parent->radius(),
-                         imgHeight2 - m_parent->radius(),
-                         2 * m_parent->radius(),
-                         2 * m_parent->radius() );
-
-    painter.restore();
-}
-
-void MarbleMapPrivate::setBoundingBox()
-{
-    QVector<QPointF>  points;
-    Quaternion        temp;
-
-    if ( m_parent->globalQuaternion( 0, 0, temp) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-    if ( m_parent->globalQuaternion( m_parent->width() / 2, 0, temp ) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-
-    if ( m_parent->globalQuaternion( m_parent->width(), 0, temp ) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-    if ( m_parent->globalQuaternion( 0, m_parent->height(), temp ) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-
-    if ( m_parent->globalQuaternion( m_parent->width()/2, m_parent->height(), temp ) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-
-    if ( m_parent->globalQuaternion( m_parent->width(), m_parent->height(), temp ) ) {
-        points.append( QPointF( temp.v[Q_X], temp.v[Q_Y]) );
-    }
-
-    m_viewParams.viewport()->setBoundingBox( BoundingBox( points ) );
-}
-
-
 // Used to be paintEvent()
 void MarbleMap::paint(GeoPainter &painter, QRect &dirtyRect) 
 {
@@ -824,7 +950,7 @@ void MarbleMap::paint(GeoPainter &painter, QRect &dirtyRect)
     customPaint( &painter );
     d->paintOverlay(painter, dirtyRect);
 
-    double fps = 1000.0 / (double)( t.elapsed() );
+    qreal fps = 1000.0 / (qreal)( t.elapsed() );
     d->paintFps(painter, dirtyRect, fps);
     emit framesPerSecond( fps );
 }
@@ -832,13 +958,17 @@ void MarbleMap::paint(GeoPainter &painter, QRect &dirtyRect)
 void MarbleMap::customPaint(GeoPainter *painter)
 {
     Q_UNUSED( painter );
-    /* This is a NOOP */
+
+    if ( !viewParams()->mapTheme() ) 
+    {
+        return;
+    }
 }
 
 void MarbleMap::goHome()
 {
-    double  homeLon = 0;
-    double  homeLat = 0;
+    qreal  homeLon = 0;
+    qreal  homeLat = 0;
     d->m_homePoint.geoCoordinates( homeLon, homeLat );
 
     centerOn( homeLon * RAD2DEG, homeLat * RAD2DEG );
@@ -893,10 +1023,27 @@ void MarbleMap::setShowAtmosphere( bool visible )
     d->doResize();
 }
 
+void MarbleMap::setShowCrosshairs( bool visible )
+{
+    QList<MarbleRenderPlugin *> pluginList = renderPlugins();
+    QList<MarbleRenderPlugin *>::const_iterator i;
+    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+        if ( (*i)->nameId() == "crosshairs" ) {
+            (*i)->setVisible( visible );
+        }
+    }
+}
+
 void MarbleMap::setShowClouds( bool visible )
 {
+    bool previousVisible = d->m_model->layerDecorator()->showClouds();
+
     d->m_model->layerDecorator()->setShowClouds( visible );
-    d->m_model->update();
+
+    if ( previousVisible != visible ) {
+        qDebug() << "Changing cloud layer";
+        d->m_model->update();
+    }
 }
 
 void MarbleMap::setShowGrid( bool visible )
@@ -972,7 +1119,7 @@ void MarbleMap::setShowGps( bool visible )
     d->m_viewParams.setShowGps( visible );
 }
 
-void MarbleMap::changeCurrentPosition( double lon, double lat)
+void MarbleMap::changeCurrentPosition( qreal lon, qreal lat)
 {
     d->m_model->gpsLayer()->changeCurrentPosition( lat, lon );
 }
@@ -980,13 +1127,13 @@ void MarbleMap::changeCurrentPosition( double lon, double lat)
 void MarbleMap::notifyMouseClick( int x, int y)
 {
     bool    valid = false;
-    double  lon   = 0;
-    double  lat   = 0;
+    qreal  lon   = 0;
+    qreal  lat   = 0;
 
-    valid = geoCoordinates( x, y, lon, lat, GeoDataPoint::Radian );
+    valid = geoCoordinates( x, y, lon, lat, GeoDataCoordinates::Radian );
 
     if ( valid ) {
-        emit mouseClickGeoPosition( lon, lat, GeoDataPoint::Radian);
+        emit mouseClickGeoPosition( lon, lat, GeoDataCoordinates::Radian);
     }
 }
 
@@ -1086,7 +1233,7 @@ void MarbleMap::setDownloadUrl( const QUrl &url )
 
 QString MarbleMap::distanceString() const
 {
-    const double VIEW_ANGLE = 110.0;
+    const qreal VIEW_ANGLE = 110.0;
 
     // Due to Marble's orthographic projection ("we have no focus")
     // it's actually not possible to calculate a "real" distance.
@@ -1098,11 +1245,24 @@ QString MarbleMap::distanceString() const
     // reality. Therefore we assume that the average window width
     // (about 800 pixels) equals the viewing angle of a human being.
     //
-    double distance = ( EARTH_RADIUS * 0.4
-			/ (double)( radius() )
+    qreal distance = ( model()->planetRadius() * 0.4
+			/ (qreal)( radius() )
 			/ tan( 0.5 * VIEW_ANGLE * DEG2RAD ) );
 
-    return QString( "%L1 %2" ).arg( distance, 8, 'f', 1, QChar(' ') ).arg( tr("km") );
+    QString distanceUnitString;
+
+    Marble::DistanceUnit distanceUnit;
+    distanceUnit = MarbleGlobal::getInstance()->locale()->distanceUnit();
+
+    if ( distanceUnit == Marble::Metric ) {
+        distanceUnitString = tr("km");
+    }
+    else {
+        distance *= KM2MI;
+        distanceUnitString = "mi";
+    }
+
+    return QString( "%L1 %2" ).arg( distance, 8, 'f', 1, QChar(' ') ).arg( distanceUnitString );
 }
 
 
@@ -1147,7 +1307,7 @@ void MarbleMap::updateSun()
     // Update the sun shading.
     //SunLocator  *sunLocator = d->m_model->sunLocator();
 
-    //qDebug() << "Updating the sun shading map...";
+    qDebug() << "MarbleMap: Updating the sun shading map...";
     d->m_model->update();
     setNeedsUpdate();
     //qDebug() << "Finished updating the sun shading map";
@@ -1157,8 +1317,8 @@ void MarbleMap::centerSun()
 {
     SunLocator  *sunLocator = d->m_model->sunLocator();
 
-    double  lon = sunLocator->getLon();
-    double  lat = sunLocator->getLat();
+    qreal  lon = sunLocator->getLon();
+    qreal  lat = sunLocator->getLat();
     centerOn( lon, lat );
 
     qDebug() << "Centering on Sun at " << lat << lon;
@@ -1169,9 +1329,9 @@ SunLocator* MarbleMap::sunLocator()
     return d->m_model->sunLocator();
 }
 
-QList<MarbleAbstractLayer *> MarbleMap::layerPlugins() const
+QList<MarbleRenderPlugin *> MarbleMap::renderPlugins() const
 {
-    return d->m_model->layerPlugins();
+    return d->m_model->renderPlugins();
 }
 
 QList<MarbleAbstractFloatItem *> MarbleMap::floatItems() const
@@ -1179,80 +1339,4 @@ QList<MarbleAbstractFloatItem *> MarbleMap::floatItems() const
     return d->m_model->floatItems();
 }
 
-void MarbleMapPrivate::paintGround( GeoPainter &painter, QRect &dirtyRect)
-{
-    if ( !m_viewParams.mapTheme() ) {
-        qDebug() << "No theme yet!";
-        return;
-    }
-
-    bool  doClip = false;
-    if ( m_viewParams.projection() == Spherical )
-        doClip = ( m_viewParams.radius() > m_viewParams.canvasImage()->width() / 2
-                   || m_viewParams.radius() > m_viewParams.canvasImage()->height() / 2 );
-
-    m_model->paintGlobe( &painter,
-                            m_parent->width(), m_parent->height(), &m_viewParams,
-                            m_parent->needsUpdate()
-                            || m_viewParams.canvasImage()->isNull(),
-                            dirtyRect );
-    // FIXME: this is ugly, add method updatePlanetAxis() to ViewParams
-    m_viewParams.setPlanetAxisUpdated( m_viewParams.planetAxis() );
-    // FIXME: this is ugly, add method updateRadius() to ViewParams
-    m_viewParams.setRadiusUpdated( m_viewParams.radius() );
-    m_justModified                   = false;
-
-    // FIXME: This is really slow. That's why we defer this to
-    //        PrintQuality. Either cache on a pixmap - or maybe
-    //        better: Add to GlobeScanlineTextureMapper.
-
-    if ( m_viewParams.mapQuality() == Marble::Print )
-        drawFog(painter);
-}
-
-void MarbleMapPrivate::paintOverlay( GeoPainter &painter, QRect &dirtyRect)
-{
-//    int transparency = ( d->m_viewParams.mapQuality() == Marble::Low ) ? 255 : 192;
-
-    // 2. Paint the crosshair.
-    m_crosshair.paint( &painter,
-                          m_viewParams.canvasImage()->width(),
-                          m_viewParams.canvasImage()->height() );
-
-    // 3. Paint measure points if there are any.
-
-    bool antialiased = false;
-
-    if (   m_viewParams.mapQuality() == Marble::High
-        || m_viewParams.mapQuality() == Marble::Print ) {
-            antialiased = true;
-    }
-
-    m_measureTool->paint( &painter, m_viewParams.viewport(), antialiased );
-
-    // Set the Bounding Box
-    setBoundingBox();
-}
-
-void MarbleMapPrivate::paintFps( GeoPainter &painter, QRect &dirtyRect, double fps)
-{
-    Q_UNUSED(dirtyRect);
-
-    if ( m_showFrameRate == true ) {
-        QString fpsString = QString( "Speed: %1 fps" ).arg( fps, 5, 'f', 1, QChar(' ') );
-
-        QPoint fpsLabelPos( 10, 20 );
-
-        painter.setFont( QFont( "Sans Serif", 10 ) );
-
-        painter.setPen( Qt::black );
-        painter.setBrush( Qt::black );
-        painter.drawText( fpsLabelPos, fpsString );
-
-        painter.setPen( Qt::white );
-        painter.setBrush( Qt::white );
-        painter.drawText( fpsLabelPos.x() - 1, fpsLabelPos.y() - 1, fpsString );
-    }
-}
-
 #include "MarbleMap.moc"
