@@ -8,15 +8,22 @@
 // Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>"
 // Copyright 2008      Jens-Michael Hoffmann <jensmh@gmx.de>
+// Copyright 2008      Patrick Spendrin <ps_ml@gmx.de>
 //
 
 #include "MarbleModel.h"
 
 #include <cmath>
 
+#if QT_VERSION < 0x040400
+# include <qatomic.h>
+#else
+# include <QtCore/QAtomicInt>
+#endif
 #include <QtCore/QDebug>
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
+#include <QtCore/QAbstractItemModel>
 #include <QtGui/QItemSelectionModel>
 #include <QtGui/QSortFilterProxyModel>
 
@@ -29,6 +36,12 @@
 #include "GeoSceneMap.h"
 #include "GeoSceneTexture.h"
 #include "GeoSceneVector.h"
+#include "GeoSceneXmlDataSource.h"
+
+#include "GeoDataDocument.h"
+#include "GeoDataStyle.h"
+
+#include "DgmlAuxillaryDictionary.h"
 
 #include "GeoPainter.h"
 #include "FileViewModel.h"
@@ -41,10 +54,11 @@
 #include "HttpDownloadManager.h"
 #include "KmlFileViewItem.h"
 #include "LayerManager.h"
-#include "MapThemeManager.h"
 #include "MarbleDataFacade.h"
 #include "MarbleDirs.h"
 #include "MarblePlacemarkModel.h"
+#include "MarbleGeometryModel.h"
+#include "MergedLayerDecorator.h"
 #include "PlaceMarkManager.h"
 #include "PlaceMarkLayout.h"
 #include "PlaceMarkPainter.h"
@@ -59,6 +73,12 @@
 #include "ViewParams.h"
 #include "ViewportParams.h"
 
+namespace Marble
+{
+
+#if QT_VERSION < 0x040400
+    typedef QAtomic QAtomicInt;
+#endif
 
 class MarbleModelPrivate
 {
@@ -74,50 +94,57 @@ class MarbleModelPrivate
     void  resize( int width, int height );
     void  notifyModelChanged();
     void  geoDataDocumentLoaded( GeoDataDocument& document );
+    void geoDataDocumentAdded( GeoDataDocument* document );
 
-    MarbleModel         *m_parent;
-    MarbleDataFacade    *m_dataFacade;
+    static QAtomicInt       refCounter;
+    MarbleModel             *m_parent;
+    MarbleDataFacade        *m_dataFacade;
 
     // View and paint stuff
-    GeoSceneDocument    *m_mapTheme;
-    LayerManager        *m_layerManager;
-    TextureColorizer    *m_texcolorizer;
+    GeoSceneDocument        *m_mapTheme;
+    LayerManager            *m_layerManager;
+    static TextureColorizer *m_texcolorizer; //left as null if unused
 
-    HttpDownloadManager *m_downloadManager;
+    HttpDownloadManager     *m_downloadManager;
 
-    TileLoader          *m_tileLoader;
+    TileLoader              *m_tileLoader;
     AbstractScanlineTextureMapper   *m_texmapper;
 
-    VectorComposer      *m_veccomposer; // FIXME: Make not a pointer.
-    GridMap             *m_gridmap;
+    static VectorComposer   *m_veccomposer; // FIXME: Make not a pointer.
+    GridMap                 *m_gridmap;
 
     // Places on the map
-    PlaceMarkManager      *m_placemarkmanager;
-    MarblePlacemarkModel  *m_placemarkmodel;
-    PlaceMarkLayout       *m_placeMarkLayout;
+    PlaceMarkManager        *m_placemarkmanager;
+    MarblePlacemarkModel    *m_placemarkmodel;
+    PlaceMarkLayout         *m_placeMarkLayout;
+    MarbleGeometryModel     *m_geometrymodel;
 
-
-    // Misc stuff
-    ExtDateTime           *m_dateTime;
-    SunLocator            *m_sunLocator;
-    MergedLayerDecorator  *m_layerDecorator;
+    // Misc stuff.
+    ExtDateTime             *m_dateTime;
+    SunLocator              *m_sunLocator;
+    MergedLayerDecorator    *m_layerDecorator;
 
     // Selection handling
-    QItemSelectionModel *m_placemarkselectionmodel;
+    QItemSelectionModel     *m_placemarkselectionmodel;
 
     //Gps Stuff
-    GpsLayer            *m_gpsLayer;
-    GpxFileModel        *m_gpxFileModel;
+    GpsLayer                *m_gpsLayer;
+    GpxFileModel            *m_gpxFileModel;
 
-    QTimer       *m_timer;
+    QTimer                  *m_timer;
 
-    FileViewModel       *m_fileviewmodel;
+    FileViewModel           *m_fileviewmodel;
 };
 
+VectorComposer      *MarbleModelPrivate::m_veccomposer = 0;
+TextureColorizer    *MarbleModelPrivate::m_texcolorizer = 0;
+QAtomicInt           MarbleModelPrivate::refCounter(0);
+
 MarbleModel::MarbleModel( QObject *parent )
     : QObject( parent ),
       d( new MarbleModelPrivate( this ) )
 {
+    MarbleModelPrivate::refCounter.ref();
     d->m_dataFacade = new MarbleDataFacade( this );
     d->m_layerManager = new LayerManager( d->m_dataFacade, this );
 
@@ -135,32 +162,37 @@ MarbleModel::MarbleModel( QObject *parent )
 
     d->m_tileLoader = new TileLoader( d->m_downloadManager, this );
 
-    connect( d->m_tileLoader, SIGNAL( paintTile(TextureTile*, int, int, int, GeoSceneTexture*, bool) ),
-             this,            SLOT( paintTile(TextureTile*, int, int, int, GeoSceneTexture*, bool) ) );
-
     d->m_texmapper = 0;
-    d->m_veccomposer = new VectorComposer();
-
-    d->m_gridmap      = new GridMap();
-    d->m_texcolorizer = new TextureColorizer( MarbleDirs::path( "seacolors.leg" ),
-                                              MarbleDirs::path( "landcolors.leg" ) );
+    d->m_gridmap      = new GridMap( this );
+    
+    if( MarbleModelPrivate::refCounter == 1 ) {
+        d->m_veccomposer = new VectorComposer();
+        /* d->m_texcolorizer is not initialized here since it takes a long time
+           to create the palette and it might not even be used. Instead it's created
+           in setMapTheme if the theme being loaded does need it. If the theme
+           doesn't need it, it's left as is. */
+    }
 
     d->m_placemarkmanager   = new PlaceMarkManager();
 
     connect( d->m_placemarkmanager, SIGNAL( geoDataDocumentLoaded( GeoDataDocument& ) ),
              this,                  SLOT( geoDataDocumentLoaded( GeoDataDocument& ) ) );
+    connect( d->m_placemarkmanager, SIGNAL( geoDataDocumentAdded( GeoDataDocument* ) ),
+             this,                  SLOT( geoDataDocumentAdded( GeoDataDocument* ) ) );
 
     d->m_placemarkmodel = new MarblePlacemarkModel( d->m_placemarkmanager, this );
     d->m_placemarkselectionmodel = new QItemSelectionModel( d->m_placemarkmodel );
+    d->m_geometrymodel = new MarbleGeometryModel();
+    d->m_placemarkmanager->setGeoModel( d->m_geometrymodel );
 
     d->m_placeMarkLayout = new PlaceMarkLayout( this );
-    connect( d->m_placemarkselectionmodel, SIGNAL( selectionChanged( QItemSelection,
-                                                                        QItemSelection) ),
-                d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
-    connect( d->m_placemarkmodel, SIGNAL( modelReset() ),
-                d->m_placeMarkLayout, SLOT( requestStyleReset() ) );
-
-    d->m_placemarkmanager->loadStandardPlaceMarks();
+    connect( d->m_placemarkmanager,         SIGNAL( finalize() ),
+             d->m_placeMarkLayout,          SLOT( requestStyleReset() ) );
+    connect( d->m_placemarkselectionmodel,  SIGNAL( selectionChanged( QItemSelection,
+                                                                      QItemSelection) ),
+             d->m_placeMarkLayout,          SLOT( requestStyleReset() ) );
+    connect( d->m_placemarkmodel,           SIGNAL( layoutChanged() ),
+             d->m_placeMarkLayout,          SLOT( requestStyleReset() ) );
 
     d->m_gpxFileModel = new GpxFileModel( this );
     d->m_gpsLayer = new GpsLayer( d->m_gpxFileModel );
@@ -184,10 +216,6 @@ MarbleModel::MarbleModel( QObject *parent )
             d->m_sunLocator, SLOT( update() ) );
     connect( d->m_layerDecorator, SIGNAL( repaintMap() ),
                                   SIGNAL( modelChanged() ) );
-
-    // TODO be able to set these somewhere
-    d->m_layerDecorator->setShowClouds( true );
-    d->m_layerDecorator->setShowTileId( false );
 }
 
 MarbleModel::~MarbleModel()
@@ -199,9 +227,13 @@ MarbleModel::~MarbleModel()
     delete d->m_tileLoader; // disconnects from downloadManager in dtor
     delete d->m_downloadManager;
 
-    delete d->m_veccomposer;
-    delete d->m_texcolorizer; 
+    if( MarbleModelPrivate::refCounter == 1 ) {
+        delete d->m_veccomposer;
+        delete d->m_texcolorizer;
+        MarbleModelPrivate::refCounter.deref();
+    }
     delete d->m_gridmap;
+    delete d->m_geometrymodel;
     delete d->m_placemarkmodel;
     delete d->m_placemarkmanager;
     delete d->m_gpsLayer;
@@ -261,7 +293,7 @@ GeoSceneDocument* MarbleModel::mapTheme() const
 //
 
 void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
-			       Projection currentProjection )
+                               Projection currentProjection )
 {
     d->m_mapTheme = mapTheme;
 
@@ -291,14 +323,16 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
         // the name of the layer that has the same name as the theme ID
         QString themeID = d->m_mapTheme->head()->theme();
 
+        GeoSceneLayer *layer = 
+            static_cast<GeoSceneLayer*>( d->m_mapTheme->map()->layer( themeID ) );
         GeoSceneTexture *texture = 
-            static_cast<GeoSceneTexture*>( d->m_mapTheme->map()->layer( themeID )->datasets().first() );
+            static_cast<GeoSceneTexture*>( layer->groundDataset() );
 
         QString sourceDir = texture->sourceDir();
         QString installMap = texture->installMap();
         QString role = d->m_mapTheme->map()->layer( themeID )->role();
 
-        if ( !TileLoader::baseTilesAvailable( texture )
+        if ( !TileLoader::baseTilesAvailable( layer )
             && !installMap.isEmpty() )
         {
             qDebug() << "Base tiles not available. Creating Tiles ... \n"
@@ -311,12 +345,12 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
                                      (role == "dem") ? "true" : "false" );
 
             TileCreatorDialog  tileCreatorDlg( tileCreator, 0 );
-	    tileCreatorDlg.setSummary( d->m_mapTheme->head()->name(),
-				       d->m_mapTheme->head()->description() );
-	    tileCreatorDlg.exec();
-	    qDebug("Tile creation completed");
+            tileCreatorDlg.setSummary( d->m_mapTheme->head()->name(),
+                                       d->m_mapTheme->head()->description() );
+            tileCreatorDlg.exec();
+            qDebug("Tile creation completed");
         }
-        d->m_tileLoader->setTextureLayer( texture );
+        d->m_tileLoader->setLayer( layer );
     }
     else {
         d->m_tileLoader->flush();
@@ -360,18 +394,47 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
         }
     }
 
-    d->m_placeMarkLayout->requestStyleReset();
-    // FIXME: To be removed after MapTheme / KML refactoring
+    d->m_geometrymodel->setGeoDataRoot( 0 );
+    QStringList loadedContainers = d->m_placemarkmanager->model()->containers();
+    QStringList loadList;
+    QVector<GeoSceneLayer*>::const_iterator it = d->m_mapTheme->map()->layers().constBegin();
+    QVector<GeoSceneLayer*>::const_iterator end = d->m_mapTheme->map()->layers().constEnd();
+    for (; it != end; ++it) {
+        GeoSceneLayer* layer = *it;
+        if ( layer->backend() == dgml::dgmlValue_geodata && layer->datasets().count() > 0 ) {
+            // look for documents
+            QVector<GeoSceneAbstractDataset*>::const_iterator itds = layer->datasets().constBegin();
+            QVector<GeoSceneAbstractDataset*>::const_iterator endds = layer->datasets().constEnd();
+            for (; itds != endds; ++itds) {
+                GeoSceneAbstractDataset* dataset = *itds;
+                if( dataset->fileFormat() == "KML" ) {
+                    loadedContainers.removeOne( reinterpret_cast<GeoSceneXmlDataSource*>(dataset)->filename() );
+                    loadList << reinterpret_cast<GeoSceneXmlDataSource*>(dataset)->filename();
+                }
+            }
+        }
+    }
+    // unload old standard Placemarks which are not part of the new map
+    foreach(const QString& container, loadedContainers) {
+        loadedContainers.pop_front();
+        d->m_placemarkmanager->model()->removePlaceMarks( container, loadedContainers.isEmpty() );
+    }
+    // load new standard Placemarks
+    foreach(const QString& container, loadList) {
+        loadList.pop_front();
+        d->m_placemarkmanager->addPlaceMarkFile( container, loadList.isEmpty() );
+    }
+    
 
     // FIXME: Still needs to get fixed for the DGML2 refactoring
 //    d->m_placeMarkLayout->placeMarkPainter()->setDefaultLabelColor( d->m_maptheme->labelColor() );
 
     if( !d->m_mapTheme->map()->filters().isEmpty() ) {
         GeoSceneFilter *filter= d->m_mapTheme->map()->filters().first();
-        QString seafile = MarbleDirs::path( "seacolors.leg" );
-        QString landfile = MarbleDirs::path( "landcolors.leg" );
 
         if( filter->type() == "colorize" ) {
+             //no need to look up with MarbleDirs twice so they are left null for now
+            QString seafile, landfile; 
             QList<GeoScenePalette*> palette = filter->palette();
             foreach ( GeoScenePalette *curPalette, palette ) {
                 if( curPalette->type() == "sea" ) {
@@ -380,14 +443,25 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
                     landfile = MarbleDirs::path( curPalette->file() );
                 }
             }
-        }
-        if( d->m_texcolorizer->seafile() != seafile ||
-            d->m_texcolorizer->landfile() != landfile ) {
-            d->m_texcolorizer->generatePalette( seafile, landfile );
+            //look up locations if they are empty
+            if(seafile.isEmpty())
+                seafile = MarbleDirs::path( "seacolors.leg" );
+            if(landfile.isEmpty())
+                landfile = MarbleDirs::path( "landcolors.leg" );
+
+            if( !d->m_texcolorizer ) {
+                /* This is where the TextureColorizer is created if it's needed
+                   by the new map theme. */
+                d->m_texcolorizer = new TextureColorizer( seafile, landfile );
+            } else if( d->m_texcolorizer->seafile() != seafile ||
+                       d->m_texcolorizer->landfile() != landfile ) {
+                d->m_texcolorizer->generatePalette( seafile, landfile );
+            }
         }
     }
     qDebug() << "THEME CHANGED: ***" << mapTheme->head()->mapThemeId();
     emit themeChanged( mapTheme->head()->mapThemeId() );
+    d->m_sunLocator->setBody(d->m_mapTheme->head()->target().toLower());
 
     d->m_layerManager->syncViewParamsAndPlugins( mapTheme );
 
@@ -413,7 +487,7 @@ void MarbleModel::setupTextureMapper( Projection projection )
     }
 
     connect( d->m_texmapper, SIGNAL( mapChanged() ),
-	     this,           SLOT( notifyModelChanged() ) );
+             this,           SLOT( notifyModelChanged() ) );
 }
 
 HttpDownloadManager* MarbleModel::downloadManager() const
@@ -448,6 +522,8 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
                               bool redrawBackground,
                               const QRect& dirtyRect )
 {
+    if ( !d->m_texmapper ) return;
+
     d->resize( width, height );
 
     // FIXME: Remove this once the LMC is there:
@@ -459,7 +535,7 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
     if ( redrawBackground ) {
         if ( d->m_mapTheme->map()->hasTextureLayers() ) {
 
-	    // Create the height map image a.k.a viewParams->m_canvasImage.
+            // Create the height map image a.k.a viewParams->m_canvasImage.
             d->m_texmapper->mapTexture( viewParams );
 
             if ( !viewParams->showElevationModel()
@@ -471,27 +547,11 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
                 // Create VectorMap
                 d->m_veccomposer->drawTextureMap( viewParams );
 
-                //set default values just in case
-                QString seafile = MarbleDirs::path( "seacolors.leg" );
-                QString landfile = MarbleDirs::path( "landcolors.leg" );
-
+                // Colorize using settings from when the map was loaded
+                // there's no need to check the palette because it's set with the map theme
                 if( filter->type() == "colorize" ) {
-                    QList<GeoScenePalette*> palette = filter->palette();
-                    foreach ( GeoScenePalette *curPalette, palette ) {
-                        if( curPalette->type() == "sea" ) {
-                            seafile = MarbleDirs::path( curPalette->file() );
-                        } else if( curPalette->type() == "land" ) {
-                            landfile = MarbleDirs::path( curPalette->file() );
-                        }
-                    }
+                    d->m_texcolorizer->colorize( viewParams );
                 }
-                if( d->m_texcolorizer->seafile() != seafile ||
-                    d->m_texcolorizer->landfile() != landfile ) {
-
-                    d->m_texcolorizer->generatePalette( seafile, landfile );
-                }
-                // Recolorize the heightmap using the VectorMap
-                d->m_texcolorizer->colorize( viewParams );
             } //else { qDebug() << "No filters to act on..."; }
         }
     }
@@ -499,12 +559,12 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
     // Paint the map on the Widget
 //    QTime t;
 //    t.start();
-    int radius = (int)(1.05 * (double)(viewParams->radius()));
+    int radius = (int)(1.05 * (qreal)(viewParams->radius()));
 
     if ( d->m_mapTheme->map()->hasTextureLayers() ) {
         if ( viewParams->projection() == Spherical ) {
             QRect rect( width / 2 - radius, height / 2 - radius,
-			2 * radius, 2 * radius);
+                        2 * radius, 2 * radius);
             rect = rect.intersect( dirtyRect );
             painter->drawImage( rect, *viewParams->canvasImage(), rect );
         }
@@ -641,6 +701,11 @@ QItemSelectionModel *MarbleModel::placeMarkSelectionModel() const
     return d->m_placemarkselectionmodel;
 }
 
+QAbstractItemModel *MarbleModel::geometryModel() const
+{
+    return d->m_geometrymodel;
+}
+
 VectorComposer    *MarbleModel::vectorComposer()   const
 {
     return d->m_veccomposer;
@@ -678,14 +743,30 @@ FileViewModel *MarbleModel::fileViewModel() const
 
 void MarbleModel::addPlaceMarkFile( const QString& filename )
 {
-    d->m_placemarkmanager->loadKml( filename, true );
+    d->m_placemarkmanager->loadKml( filename, false );
 
     d->notifyModelChanged();
 }
 
-void MarbleModel::addPlaceMarkData( const QString& data )
+void MarbleModel::addPlaceMarkData( const QString& data, const QString& key )
 {
-    d->m_placemarkmanager->loadKmlFromData( data, true );
+    QStringList loadedContainers = d->m_placemarkmanager->model()->containers();
+    if( loadedContainers.contains( key ) )
+    {
+        d->m_placemarkmanager->model()->removePlaceMarks( key, false );
+    }
+    d->m_placemarkmanager->loadKmlFromData( data, key, false );
+
+    d->notifyModelChanged();
+}
+
+void MarbleModel::removePlaceMarkKey( const QString& key )
+{
+    QStringList loadedContainers = d->m_placemarkmanager->model()->containers();
+    if( loadedContainers.contains( key ) )
+    {
+        d->m_placemarkmanager->model()->removePlaceMarks( key, false );
+    }
 
     d->notifyModelChanged();
 }
@@ -708,11 +789,70 @@ void MarbleModelPrivate::geoDataDocumentLoaded( GeoDataDocument& document )
     m_fileviewmodel->append( item );
 }
 
+void MarbleModelPrivate::geoDataDocumentAdded( GeoDataDocument* document )
+{
+    AbstractFileViewItem* item = new KmlFileViewItem( *m_placemarkmanager,
+                                                      *document );
+
+    foreach(GeoDataPlacemark* placemark, document->placemarks())
+    {
+        QString styleUrl = placemark->styleUrl().remove('#');
+        placemark->setStyle( document->style( styleUrl ) );
+    };
+    m_fileviewmodel->append( item );
+    
+    m_geometrymodel->setGeoDataRoot( document );
+}
+
 void MarbleModel::update()
 {
+    qDebug() << "MarbleModel::update()";
     QTimer::singleShot( 0, d->m_tileLoader, SLOT( update() ) );
 }
 
+qreal MarbleModel::planetRadius()   const
+{
+    if ( d->m_mapTheme ) {
+        QString target = d->m_mapTheme->head()->target().toLower();
+
+        // I'd like to be able to use strings in switch statements ... 
+
+        // Make sure to use the radius, not the diameter!
+
+        // planets
+               if ( target == "mercury" ) {
+            return  2440000.0;
+        } else if ( target == "venus" ) {
+            return  6051800.0;
+        } else if ( target == "earth" ) {
+            return  6378000.0;
+        } else if ( target == "mars" ) {
+            return  3397000.0;
+        } else if ( target == "jupiter" ) {
+            return  71492000.0;
+        } else if ( target == "saturn" ) {
+            return  60268000.0;
+        } else if ( target == "uranus" ) {
+            return  25559000.0;
+        } else if ( target == "neptune" ) {
+            return  24766000.0;
+
+        // dwarf planets ... (everybody likes pluto)
+        } else if ( target == "pluto" ) {
+            return  1151000.0;
+
+        // sun and moon
+        } else if ( target == "sun" ) {
+            return  695000000.0;
+        } else if ( target == "moon" ) {
+            return  1738000.0;
+        }
+    }
+
+    // Fallback to assuming that we deal with the earth
+    return 6378000.0;
+}
+
 ExtDateTime* MarbleModel::dateTime() const
 {
 //    qDebug() << "In dateTime, model:" << this;
@@ -759,14 +899,16 @@ void MarbleModel::clearPersistentTileCache()
         // the name of the layer that has the same name as the theme ID
         QString themeID = d->m_mapTheme->head()->theme();
 
+        GeoSceneLayer *layer = 
+            static_cast<GeoSceneLayer*>( d->m_mapTheme->map()->layer( themeID ) );
         GeoSceneTexture *texture = 
-            static_cast<GeoSceneTexture*>( d->m_mapTheme->map()->layer( themeID )->datasets().first() );
+            static_cast<GeoSceneTexture*>( layer->groundDataset() );
 
         QString sourceDir = texture->sourceDir();
         QString installMap = texture->installMap();
         QString role = d->m_mapTheme->map()->layer( themeID )->role();
 
-        if ( !TileLoader::baseTilesAvailable( texture )
+        if ( !TileLoader::baseTilesAvailable( layer )
             && !installMap.isEmpty() )
         {
             qDebug() << "Base tiles not available. Creating Tiles ... \n"
@@ -804,9 +946,9 @@ void MarbleModel::paintTile(TextureTile* tile, int x, int y, int level,
     tile->loadTile(requestTileUpdate);
 }
 
-QList<MarbleAbstractLayer *> MarbleModel::layerPlugins() const
+QList<MarbleRenderPlugin *> MarbleModel::renderPlugins() const
 {
-    return d->m_layerManager->layerPlugins();
+    return d->m_layerManager->renderPlugins();
 }
 
 QList<MarbleAbstractFloatItem *> MarbleModel::floatItems() const
@@ -814,4 +956,6 @@ QList<MarbleAbstractFloatItem *> MarbleModel::floatItems() const
     return d->m_layerManager->floatItems();
 }
 
+}
+
 #include "MarbleModel.moc"
