@@ -11,8 +11,10 @@
 
 #include "MarbleThemeSelectView.h"
 
+#include <QtCore/QDebug>
 #include <QtGui/QResizeEvent>
 
+using namespace Marble;
 
 MarbleThemeSelectView::MarbleThemeSelectView(QWidget *parent)
     : QListView( parent ),
