@@ -14,11 +14,13 @@
 #include <cmath>
 
 #include <QtCore/QCoreApplication>
+#include <QtCore/QDebug>
 #include <QtCore/QLocale>
 #include <QtCore/QTranslator>
 #include <QtCore/QAbstractItemModel>
 #include <QtGui/QItemSelectionModel>
 #include <QtGui/QSizePolicy>
+#include <QtGui/QPaintEvent>
 #include <QtGui/QRegion>
 #include <QtGui/QStyleOptionGraphicsItem>
 #include <QtNetwork/QNetworkProxy>
@@ -33,7 +35,7 @@
 #include "ViewParams.h"
 #include "GeoPainter.h"
 #include "FileViewModel.h"
-#include "GeoDataPoint.h"
+#include "GeoDataCoordinates.h"
 #include "GpxFileViewItem.h"
 #include "MarblePhysics.h"
 #include "MarblePlacemarkModel.h"
@@ -52,6 +54,8 @@
 
 #include "MarbleMap_p.h"
 
+namespace Marble
+{
 
 #ifdef Q_CC_MSVC
 # ifndef KDEWIN_MATH_H
@@ -60,11 +64,6 @@
 #endif
 
 
-// FIXME(IW): Why is the construct function in
-//            MarbleWidgetPrivate??  That class is only supposed to
-//            hold data for binary compatibility, not have functions.
-
-
 class MarbleWidgetPrivate
 {
  public:
@@ -90,8 +89,6 @@ class MarbleWidgetPrivate
 
     void  construct();
 
-    void  paintEmptyMapWarning();
-
     MarbleWidget    *m_widget;
     // The model we are showing.
     MarbleMap       *m_map;
@@ -136,8 +133,6 @@ MarbleWidget::MarbleWidget(MarbleMap *map, QWidget *parent)
 //    QDBusConnection::sessionBus().registerObject("/marble", this, QDBusConnection::QDBusConnection::ExportAllSlots);
 
     d->construct();
-    
-    setMapThemeId("earth/srtm/srtm.dgml");
 }
 
 MarbleWidget::~MarbleWidget()
@@ -198,8 +193,8 @@ void MarbleWidgetPrivate::construct()
     // The interface to the measure tool consists of a RMB popup menu
     // and some signals.
     MeasureTool  *measureTool = m_map->measureTool();
-    m_widget->connect( m_popupmenu, SIGNAL( addMeasurePoint( double, double ) ),
-                       measureTool, SLOT( addMeasurePoint( double, double ) ) );
+    m_widget->connect( m_popupmenu, SIGNAL( addMeasurePoint( qreal, qreal ) ),
+                       measureTool, SLOT( addMeasurePoint( qreal, qreal ) ) );
     m_widget->connect( m_popupmenu, SIGNAL( removeMeasurePoints() ),
                        measureTool, SLOT( removeMeasurePoints( ) ) );
 
@@ -215,12 +210,6 @@ void MarbleWidgetPrivate::construct()
 
     m_widget->goHome();
 
-    // Widget translation
-    QString      locale = QLocale::system().name();
-    QTranslator  translator;
-    translator.load(QString("marblewidget_") + locale);
-    QCoreApplication::installTranslator(&translator);
-
     m_widget->connect( m_model->sunLocator(), SIGNAL( reenableWidgetInput() ),
                        m_widget, SLOT( enableInput() ) );
 
@@ -231,32 +220,6 @@ void MarbleWidgetPrivate::construct()
                         m_widget, SLOT( updateAnimation( qreal ) ) );
 }
 
-void  MarbleWidgetPrivate::paintEmptyMapWarning()
-{
-    QPainter logoPainter;
-    logoPainter.begin( m_widget );
-
-    QPixmap logoPixmap( MarbleDirs::path( "svg/marble-logo-inverted-72dpi.png" ) );
-
-    if ( logoPixmap.width() > m_widget->width() * 0.7 || logoPixmap.height() > m_widget->height() * 0.7 )
-    {
-        logoPixmap = logoPixmap.scaled( m_widget->size() * 0.7, Qt::KeepAspectRatio, Qt::SmoothTransformation );
-    }
-
-    QPoint logoPosition( ( m_widget->width()  - logoPixmap.width() ) / 2 , 
-                            ( m_widget->height() - logoPixmap.height() ) / 2 ); 
-    logoPainter.drawPixmap( logoPosition, logoPixmap );
-
-    QString noThemeWarning = "Please assign a map theme!";
-
-    logoPainter.setPen( Qt::white );
-    QRect textRect( 0, logoPosition.y() + logoPixmap.height() + m_widget->fontMetrics().height(),
-                    m_widget->width(), m_widget->fontMetrics().height() );
-    logoPainter.drawText( textRect, Qt::AlignCenter, noThemeWarning ); 
-    logoPainter.end();
-}
-
-
 // ----------------------------------------------------------------
 
 
@@ -311,7 +274,7 @@ int MarbleWidget::radius() const
     return d->m_map->radius();
 }
 
-void MarbleWidget::setRadius(const int radius)
+void MarbleWidget::setRadius(int radius)
 {
     d->m_map->setRadius( radius );
 
@@ -346,7 +309,7 @@ QItemSelectionModel *MarbleWidget::placeMarkSelectionModel() const
     return d->m_map->placeMarkSelectionModel();
 }
 
-double MarbleWidget::moveStep()
+qreal MarbleWidget::moveStep()
 {
     return d->m_map->moveStep();
 }
@@ -372,9 +335,14 @@ void MarbleWidget::addPlaceMarkFile( const QString &filename )
     //d->m_model->addPlaceMarkFile( filename );
 }
 
-void MarbleWidget::addPlaceMarkData( const QString &data )
+void MarbleWidget::addPlaceMarkData( const QString &data, const QString &key )
 {
-    d->m_map->addPlaceMarkData( data );
+    d->m_map->addPlaceMarkData( data, key );
+}
+
+void MarbleWidget::removePlaceMarkKey( const QString &key )
+{
+    d->m_map->removePlaceMarkKey( key );
 }
 
 QPixmap MarbleWidget::mapScreenShot()
@@ -407,6 +375,11 @@ bool MarbleWidget::showAtmosphere() const
     return d->m_map->showAtmosphere();
 }
 
+bool MarbleWidget::showCrosshairs() const
+{
+    return d->m_map->showCrosshairs();
+}
+
 bool MarbleWidget::showGrid() const
 {
     return d->m_map->showGrid();
@@ -536,7 +509,7 @@ void MarbleWidget::rotateBy(const Quaternion& incRot)
     repaint();
 }
 
-void MarbleWidget::rotateBy( const double& deltaLon, const double& deltaLat)
+void MarbleWidget::rotateBy( const qreal& deltaLon, const qreal& deltaLat)
 {
     d->m_map->rotateBy( deltaLon, deltaLat );
 
@@ -548,11 +521,12 @@ void MarbleWidget::rotateBy( const double& deltaLon, const double& deltaLat)
 }
 
 
-void MarbleWidget::centerOn( const double& lon, const double& lat, bool animated )
+void MarbleWidget::centerOn( const qreal& lon, const qreal& lat, bool animated )
 {
     if ( d->m_animationsEnabled && animated )
     {
-        d->m_physics->jumpTo( GeoDataPoint( lon, lat, distance(), GeoDataPoint::Degree ) );
+        d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(), distance(), GeoDataCoordinates::Degree ) );        
+        d->m_physics->jumpTo( GeoDataCoordinates( lon, lat, distance(), GeoDataCoordinates::Degree ) );
     }
     else
     {
@@ -576,9 +550,9 @@ void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
         selectionModel->clear();
     
         if ( index.isValid() ) {
-            const GeoDataPoint targetPosition = index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataPoint>();
+            const GeoDataCoordinates targetPosition = index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
 
-            d->m_physics->setCurrentPosition( GeoDataPoint( centerLongitude(), centerLatitude(), distance(), GeoDataPoint::Degree ) );
+            d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(), distance(), GeoDataCoordinates::Degree ) );
             d->m_physics->jumpTo( targetPosition );
 
             selectionModel->select( index, QItemSelectionModel::SelectCurrent );
@@ -596,19 +570,19 @@ void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
     repaint();
 }
 
-void MarbleWidget::centerOn( const GeoDataPoint &position, bool animated )
+void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 {
     if ( d->m_animationsEnabled && animated )
     {
-        GeoDataPoint targetPosition = position;
+        GeoDataCoordinates targetPosition = position;
         targetPosition.setAltitude( distance() );
 
         d->m_physics->jumpTo( targetPosition );
     }
     else
     {
-        double  lon, lat;
-        position.geoCoordinates( lon, lat, GeoDataPoint::Degree );
+        qreal  lon, lat;
+        position.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
         d->m_map->setDistance( position.altitude() );
         d->m_map->centerOn( lon, lat );
     }
@@ -622,7 +596,7 @@ void MarbleWidget::centerOn( const GeoDataPoint &position, bool animated )
 
 void MarbleWidget::updateAnimation( qreal updateValue )
 {
-    GeoDataPoint position = d->m_physics->suggestedPosition();
+    GeoDataCoordinates position = d->m_physics->suggestedPosition();
 
     if ( updateValue < 1.0 )
     {
@@ -635,7 +609,7 @@ void MarbleWidget::updateAnimation( qreal updateValue )
     centerOn( position );
 }
 
-void MarbleWidget::setCenterLatitude( double lat )
+void MarbleWidget::setCenterLatitude( qreal lat )
 {
     centerOn( centerLongitude(), lat );
 
@@ -644,7 +618,7 @@ void MarbleWidget::setCenterLatitude( double lat )
     setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 }
 
-void MarbleWidget::setCenterLongitude( double lon )
+void MarbleWidget::setCenterLongitude( qreal lon )
 {
     centerOn( lon, centerLatitude() );
 
@@ -672,17 +646,17 @@ void MarbleWidget::setProjection( int projection )
     setProjection( (Projection)( projection ) );
 }
 
-void MarbleWidget::home( double &lon, double &lat, int& zoom )
+void MarbleWidget::home( qreal &lon, qreal &lat, int& zoom )
 {
     d->m_map->home( lon, lat, zoom );
 }
 
-void MarbleWidget::setHome( const double lon, const double lat, const int zoom)
+void MarbleWidget::setHome( qreal lon, qreal lat, int zoom )
 {
     d->m_map->setHome( lon, lat, zoom );
 }
 
-void MarbleWidget::setHome(const GeoDataPoint& homePoint, int zoom)
+void MarbleWidget::setHome(const GeoDataCoordinates& homePoint, int zoom)
 {
     d->m_map->setHome( homePoint, zoom );
 }
@@ -768,25 +742,25 @@ int MarbleWidget::northPoleY()
     return d->m_map->northPoleY();
 }
 
-bool MarbleWidget::screenCoordinates( const double lon, const double lat,
+bool MarbleWidget::screenCoordinates( qreal lon, qreal lat,
                                       int& x, int& y )
 {
     return d->m_map->screenCoordinates( lon, lat, x, y );
 }
 
-bool MarbleWidget::geoCoordinates(const int x, const int y,
-                                  double& lon, double& lat,
-                                  GeoDataPoint::Unit unit )
+bool MarbleWidget::geoCoordinates(int x, int y,
+                                  qreal& lon, qreal& lat,
+                                  GeoDataCoordinates::Unit unit )
 {
     return d->m_map->geoCoordinates( x, y, lon, lat, unit );
 }
 
-double MarbleWidget::centerLatitude() const
+qreal MarbleWidget::centerLatitude() const
 {
     return d->m_map->centerLatitude();
 }
 
-double MarbleWidget::centerLongitude() const
+qreal MarbleWidget::centerLongitude() const
 {
     return d->m_map->centerLongitude();
 }
@@ -796,17 +770,17 @@ bool MarbleWidget::globalQuaternion( int x, int y, Quaternion &q)
     int  imageHalfWidth  = width() / 2;
     int  imageHalfHeight = height() / 2;
 
-    const double  inverseRadius = 1.0 / (double)(radius());
+    const qreal  inverseRadius = 1.0 / (qreal)(radius());
 
-    if ( radius() > sqrt( (double)(( x - imageHalfWidth ) * ( x - imageHalfWidth )
+    if ( radius() > sqrt( (qreal)(( x - imageHalfWidth ) * ( x - imageHalfWidth )
         + ( y - imageHalfHeight ) * ( y - imageHalfHeight )) ) )
     {
-        double qx = inverseRadius * (double)( x - imageHalfWidth );
-        double qy = inverseRadius * (double)( y - imageHalfHeight );
-        double qr = 1.0 - qy * qy;
+        qreal qx = inverseRadius * (qreal)( x - imageHalfWidth );
+        qreal qy = inverseRadius * (qreal)( y - imageHalfHeight );
+        qreal qr = 1.0 - qy * qy;
 
-        double qr2z = qr - qx * qx;
-        double qz = ( qr2z > 0.0 ) ? sqrt( qr2z ) : 0.0;
+        qreal qr2z = qr - qx * qx;
+        qreal qz = ( qr2z > 0.0 ) ? sqrt( qr2z ) : 0.0;
 
         Quaternion  qpos( 0.0, qx, qy, qz );
         qpos.rotateAroundAxis( planetAxis() );
@@ -830,12 +804,6 @@ const QRegion MarbleWidget::projectedRegion()
 
 void MarbleWidget::paintEvent(QPaintEvent *evt)
 {
-    if ( mapThemeId().isEmpty() ) 
-    {
-        d->paintEmptyMapWarning();
-        return;
-    }
-
     QTime t;
     t.start();
 
@@ -857,7 +825,7 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
     customPaint(&painter);
     d->m_map->d->paintOverlay( painter, dirtyRect );
 
-    double fps = 1000.0 / (double)( t.elapsed() );
+    qreal fps = 1000.0 / (qreal)( t.elapsed() );
     d->m_map->d->paintFps(painter, dirtyRect, fps);
     emit d->m_map->framesPerSecond( fps );
 }
@@ -948,6 +916,13 @@ void MarbleWidget::setShowAtmosphere( bool visible )
     repaint();
 }
 
+void MarbleWidget::setShowCrosshairs( bool visible )
+{
+    d->m_map->setShowCrosshairs( visible );
+
+    repaint();
+}
+
 void MarbleWidget::setShowGrid( bool visible )
 {
     d->m_map->setShowGrid( visible );
@@ -1039,7 +1014,7 @@ void MarbleWidget::setShowGps( bool visible )
     repaint();
 }
 
-void MarbleWidget::changeCurrentPosition( double lon, double lat)
+void MarbleWidget::changeCurrentPosition( qreal lon, qreal lat)
 {
     d->m_model->gpsLayer()->changeCurrentPosition( lat, lon );
     repaint();
@@ -1048,13 +1023,13 @@ void MarbleWidget::changeCurrentPosition( double lon, double lat)
 void MarbleWidget::notifyMouseClick( int x, int y)
 {
     bool    valid = false;
-    double  lon   = 0;
-    double  lat   = 0;
+    qreal  lon   = 0;
+    qreal  lat   = 0;
 
-    valid = geoCoordinates( x, y, lon, lat, GeoDataPoint::Radian );
+    valid = geoCoordinates( x, y, lon, lat, GeoDataCoordinates::Radian );
 
     if ( valid ) {
-        emit mouseClickGeoPosition( lon, lat, GeoDataPoint::Radian);
+        emit mouseClickGeoPosition( lon, lat, GeoDataCoordinates::Radian);
     }
 }
 
@@ -1230,19 +1205,41 @@ void MarbleWidget::setDefaultFont( const QFont& font )
     map()->setDefaultFont( font );
 }
 
-double MarbleWidget::distance() const
+void MarbleWidget::setSelection(const QRect& region)
+{
+    QPoint tl = region.topLeft();
+    QPoint br = region.bottomRight();
+    qDebug() << "Selection region: (" << tl.x() << ", " <<  tl.y() << ") (" 
+             << br.x() << ", " << br.y() << ")" << endl;
+
+    AbstractProjection *proj = d->m_map->viewParams()->currentProjection();
+    GeoDataLatLonAltBox box  = proj->latLonAltBox(region, d->m_map->viewParams()->viewport());
+
+    // NOTE: coordinates as lon1, lat1, lon2, lat2 (or West, North, East, South)
+    // as left/top, right/bottom rectangle.
+    QList<double> coordinates;
+    coordinates << box.west(GeoDataPoint::Degree) << box.north(GeoDataPoint::Degree) 
+                << box.east(GeoDataPoint::Degree) << box.south(GeoDataPoint::Degree);
+
+    qDebug() << "West: " << coordinates[0] << " North: " <<  coordinates[1] 
+             << " East: " << coordinates[2] << " South: " << coordinates[3] << endl;
+
+    emit regionSelected(coordinates);
+}
+
+qreal MarbleWidget::distance() const
 {
     return map()->distance();
 }
 
-void MarbleWidget::setDistance( double distance )
+void MarbleWidget::setDistance( qreal distance )
 {
     map()->setDistance( distance );
 }
 
 QString MarbleWidget::distanceString() const
 {
-    return QString( "%L1 %2" ).arg( distance(), 8, 'f', 1, QChar(' ') ).arg( tr("km") );
+    return map()->distanceString();
 }
 
 void MarbleWidget::updateSun()
@@ -1250,7 +1247,7 @@ void MarbleWidget::updateSun()
     // Update the sun shading.
     //SunLocator  *sunLocator = d->m_model->sunLocator();
 
-    //qDebug() << "Updating the sun shading map...";
+    qDebug() << "MarbleWidget: Updating the sun shading map...";
     d->m_model->update();
     setNeedsUpdate();
     //qDebug() << "Finished updating the sun shading map";
@@ -1260,8 +1257,8 @@ void MarbleWidget::centerSun()
 {
     SunLocator  *sunLocator = d->m_model->sunLocator();
 
-    double  lon = sunLocator->getLon();
-    double  lat = sunLocator->getLat();
+    qreal  lon = sunLocator->getLon();
+    qreal  lat = sunLocator->getLat();
     centerOn( lon, lat );
 
     qDebug() << "centering on Sun at" << lat << lon;
@@ -1312,13 +1309,16 @@ quint16 MarbleWidget::proxyPort() const
     return d->m_proxyPort;
 }
 
-QList<MarbleAbstractLayer *> MarbleWidget::layerPlugins() const
+QList<MarbleRenderPlugin *> MarbleWidget::renderPlugins() const
 {
-    return d->m_model->layerPlugins();
+    return d->m_model->renderPlugins();
 }
 
 QList<MarbleAbstractFloatItem *> MarbleWidget::floatItems() const
 {
     return d->m_model->floatItems();
 }
+
+}
+
 #include "MarbleWidget.moc"
