@@ -31,7 +31,7 @@
 //#include <QtDBus/QtDBus>
 
 #include "marble_export.h"
-#include "GeoDataPoint.h"       // In geodata/data/
+#include "GeoDataCoordinates.h"       // In geodata/data/
 #include "MarbleWidgetInputHandler.h"
 #include "global.h"             // types needed in all of marble.
 
@@ -41,7 +41,8 @@ class QModelIndex;
 class QItemSelectionModel;
 class QStyleOptionGraphicsItem;
 
-class GeoSceneDocument;
+namespace Marble
+{
 
 // MarbleWidget 
 // class MarbleWidgetInputHandler;
@@ -57,9 +58,9 @@ class SunLocator;
 class GpxFileModel;
 class FileViewModel;
 class GeoPainter;
-class MarbleAbstractLayer;
+class MarbleRenderPlugin;
 class MarbleAbstractFloatItem;
-
+class GeoSceneDocument;
 
 /**
  * @short A widget class that displays a view of the earth.
@@ -121,15 +122,17 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     Q_PROPERTY(QString mapThemeId  READ mapThemeId    WRITE setMapThemeId)
     Q_PROPERTY(int projection    READ projection      WRITE setProjection)
 
-    Q_PROPERTY(double longitude  READ centerLongitude WRITE setCenterLongitude)
-    Q_PROPERTY(double latitude   READ centerLatitude  WRITE setCenterLatitude)
+    Q_PROPERTY(qreal longitude  READ centerLongitude WRITE setCenterLongitude)
+    Q_PROPERTY(qreal latitude   READ centerLatitude  WRITE setCenterLatitude)
 
+    Q_PROPERTY(bool showOverviewMap READ showOverviewMap    WRITE setShowOverviewMap)
     Q_PROPERTY(bool showScaleBar READ showScaleBar    WRITE setShowScaleBar)
     Q_PROPERTY(bool showCompass  READ showCompass     WRITE setShowCompass)
     Q_PROPERTY(bool showGrid     READ showGrid        WRITE setShowGrid)
 
     Q_PROPERTY(bool showClouds   READ showClouds      WRITE setShowClouds)
     Q_PROPERTY(bool showAtmosphere READ showAtmosphere WRITE setShowAtmosphere)
+    Q_PROPERTY(bool showCrosshairs READ showCrosshairs WRITE setShowCrosshairs)
 
     Q_PROPERTY(bool showPlaces   READ showPlaces      WRITE setShowPlaces)
     Q_PROPERTY(bool showCities   READ showCities      WRITE setShowCities)
@@ -215,7 +218,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @brief  Set the radius of the globe in pixels.
      * @param  radius  The new globe radius value in pixels.
      */
-    void        setRadius(const int radius);
+    void        setRadius(int radius);
 
     /**
      * @brief Return the current zoom level.
@@ -225,13 +228,13 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief Return the current distance.
      */
-    double      distance() const;
+    qreal      distance() const;
 
     /**
      * @brief  Set the distance of the observer to the globe in km.
      * @param  distance  The new distance in km.
      */
-    void        setDistance( double distance );
+    void        setDistance( qreal distance );
 
     /**
      * @brief Return the current distance string.
@@ -266,7 +269,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @return @c true  if the geographical coordinates are visible on the screen
      *         @c false if the geographical coordinates are not visible on the screen
      */
-    bool screenCoordinates( const double lon, const double lat,
+    bool screenCoordinates( qreal lon, qreal lat,
                             int& x, int& y );
 
     /**
@@ -284,9 +287,9 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @return @c true  if the pixel (x, y) is within the globe
      *         @c false if the pixel (x, y) is outside the globe, i.e. in space.
      */
-    bool geoCoordinates( const int x, const int y,
-                         double& lon, double& lat,
-                         GeoDataPoint::Unit = GeoDataPoint::Degree );
+    bool geoCoordinates( int x, int y,
+                         qreal& lon, qreal& lat,
+                         GeoDataCoordinates::Unit = GeoDataCoordinates::Degree );
 
     /**
      * @brief Get a quaternion representing a point on the earth corresponding to a pixel in the widget.
@@ -303,12 +306,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief Return the longitude of the center point.
      */
-    double  centerLongitude() const;
+    qreal  centerLongitude() const;
 
     /**
      * @brief Return the latitude of the center point.
      */
-    double  centerLatitude()  const;
+    qreal  centerLatitude()  const;
 
     /**
      * @brief Returns the model for all the placemarks on the globe.
@@ -324,7 +327,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @brief  Return how much the map will move if one of the move slots are called.
      * @return The move step.
      */
-    double  moveStep();
+    qreal  moveStep();
 
     /**
      * @brief  Add a GeoDataPlacemark file to the model.
@@ -335,8 +338,15 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief  Add GeoDataPlacemark data as string to the model.
      * @param  data  the string containing the PlaceMarks.
+     * @param key  the string needed to identify the data
      */
-    void addPlaceMarkData( const QString &data );
+    void addPlaceMarkData( const QString& data, const QString& key = "data" );
+    
+    /**
+     * @brief  remove data or files from the model.
+     * @param key  either the filename or the string used to identify the data in addPlaceMarkFile and addPlaceMarkData
+     */
+    void removePlaceMarkKey( const QString& key );
 
     /**
      * @brief  Return the quaternion that specifies the rotation of the globe.
@@ -349,6 +359,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     QPixmap mapScreenShot();
 
+    /**
+     * @brief  Return whether the overview map is visible.
+     * @return The overview map visibility.
+     */
+    bool  showOverviewMap() const;
+
     /**
      * @brief  Return whether the scale bar is visible.
      * @return The scale bar visibility.
@@ -373,6 +389,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool  showAtmosphere() const;
 
+    /**
+     * @brief  Return whether the crosshairs are visible.
+     * @return The crosshairs' visibility.
+     */
+    bool  showCrosshairs() const;
+
     /**
      * @brief  Return whether the coordinate grid is visible.
      * @return The coordinate grid visibility.
@@ -469,8 +491,65 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     QString proxyHost() const;
     quint16 proxyPort() const;
 
-    QList<MarbleAbstractLayer *> layerPlugins() const;
-    QList<MarbleAbstractFloatItem *> floatItems() const;
+    QList<MarbleRenderPlugin *>      renderPlugins() const;
+    QList<MarbleAbstractFloatItem *> floatItems()    const;
+
+
+    /**
+     * @brief  Get the Projection used for the map
+     * @return @c Spherical         a Globe
+     * @return @c Equirectangular   a flat map
+     * @return @c Mercator          another flat map
+     */
+    Projection  projection() const;
+//    int         projection() const;
+
+    /**
+     * @brief Get the ID of the current map theme
+     * To ensure that a unique identifier is being used the theme does NOT 
+     * get represented by its name but the by relative location of the file 
+     * that specifies the theme:
+     *
+     * Example: 
+     *    mapThemeId = "earth/bluemarble/bluemarble.dgml"
+     */
+    QString mapThemeId() const;
+
+    /**
+     * @brief Get the GeoSceneDocument object of the current map theme
+     */
+    Marble::GeoSceneDocument *mapTheme() const;
+
+    /**
+     * @brief Return a QAbstractItemModel containing GPX files.
+     */
+    GpxFileModel  *gpxFileModel();
+
+    /**
+     * @brief Return a QAbstractItemModel containing files.
+     */
+    FileViewModel* fileViewModel() const;
+
+    /**
+     * @brief Retrieve the map quality depending on the view context 
+     */
+    Marble::MapQuality mapQuality( Marble::ViewContext = Marble::Still );
+
+    /**
+     * @brief Retrieve the view context (i.e. still or animated map) 
+     */
+    Marble::ViewContext viewContext( ) const;
+
+    /**
+     * @brief Retrieve whether travels to a point should get animated 
+     */
+    bool animationsEnabled() const;
+
+    Marble::AngleUnit defaultAngleUnit() const;
+    void setDefaultAngleUnit( Marble::AngleUnit angleUnit );
+
+    QFont defaultFont() const;
+    void setDefaultFont( const QFont& font );
 
  public Q_SLOTS:
 
@@ -516,7 +595,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * of (lon, lat), otherwise the resulting angle will be the sum of
      * the previous position and the two offsets.
      */
-    void  rotateBy( const double &deltaLon, const double &deltaLat );
+    void  rotateBy( const qreal &deltaLon, const qreal &deltaLat );
 
     /**
      * @brief  Rotate the view by the angle specified by a Quaternion.
@@ -531,7 +610,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @param  lon  an angle parallel to the longitude lines
      *              +180(W) - -180(E)
      */
-    void  centerOn( const double &lon, const double &lat, bool animated = false );
+    void  centerOn( const qreal &lon, const qreal &lat, bool animated = false );
 
     /**
      * @brief  Center the view on a point
@@ -544,28 +623,19 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @param  point the point above earth from which the view can be seen
      *               by looking vertically down.
      */
-    void  centerOn( const GeoDataPoint &point, bool animated = false );
+    void  centerOn( const GeoDataCoordinates &point, bool animated = false );
 
     /**
      * @brief  Set the latitude for the center point
      * @param  lat  the new value for the latitude
      */
-    void setCenterLatitude( double lat );
+    void setCenterLatitude( qreal lat );
 
     /**
      * @brief  Set the longitude for the center point
      * @param  lon  the new value for the longitude
      */
-    void setCenterLongitude( double lon );
-
-    /**
-     * @brief  Get the Projection used for the map
-     * @return @c Spherical         a Globe
-     * @return @c Equirectangular   a flat map
-     * @return @c Mercator          another flat map
-     */
-    Projection  projection() const;
-//    int         projection() const;
+    void setCenterLongitude( qreal lon );
 
     /**
      * @brief  Set the Projection used for the map
@@ -580,20 +650,20 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @param  lat  the latitude of the home point.
      * @param  zoom the default zoom level of the home point.
      */
-    void  home(double &lon, double &lat, int& zoom);
+    void  home(qreal &lon, qreal &lat, int& zoom);
     /**
      * @brief  Set the home point
      * @param  lon  the longitude of the new home point.
      * @param  lat  the latitude of the new home point.
      * @param  zoom the default zoom level for the new home point.
      */
-    void  setHome(const double lon, const double lat, const int zoom = 1050);
+    void  setHome(qreal lon, qreal lat, int zoom = 1050);
     /**
      * @brief  Set the home point
      * @param  homePoint  the new home point.
      * @param  zoom       the default zoom level for the new home point.
      */
-    void  setHome(const GeoDataPoint& homePoint, int zoom = 1050);
+    void  setHome(const GeoDataCoordinates& homePoint, int zoom = 1050);
 
     /**
      * @brief  Move left by the moveStep.
@@ -617,22 +687,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void  goHome();
 
-    /**
-     * @brief Get the ID of the current map theme
-     * To ensure that a unique identifier is being used the theme does NOT 
-     * get represented by its name but the by relative location of the file 
-     * that specifies the theme:
-     *
-     * Example: 
-     *    mapThemeId = "earth/bluemarble/bluemarble.dgml"
-     */
-    QString mapThemeId() const;
-
-    /**
-     * @brief Get the GeoSceneDocument object of the current map theme
-     */
-    GeoSceneDocument *mapTheme() const;
-
     /**
      * @brief Set a new map theme
      * @param maptheme  The ID of the new maptheme. To ensure that a unique 
@@ -653,6 +707,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setPropertyValue( const QString& name, bool value );
 
+    /**
+     * @brief  Set whether the overview map overlay is visible
+     * @param  visible  visibility of the overview map
+     */
+    void setShowOverviewMap( bool visible );
+
     /**
      * @brief  Set whether the scale bar overlay is visible
      * @param  visible  visibility of the scale bar
@@ -677,6 +737,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowAtmosphere( bool visible );
 
+    /**
+     * @brief  Set whether the crosshairs are visible
+     * @param  visible  visibility of the crosshairs
+     */
+    void setShowCrosshairs( bool visible );
+
     /**
      * @brief  Set whether the coordinate grid overlay is visible
      * @param  visible  visibility of the coordinate grid
@@ -760,7 +826,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @param lat the new latitude value
      * @param lon the new longitude value
      */
-    void changeCurrentPosition( double lon, double lat );
+    void changeCurrentPosition( qreal lon, qreal lat );
 
      /**
      * @brief Used to notify about the position of the mouse click
@@ -777,16 +843,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void openGpxFile( QString &filename );
 
-    /**
-     * @brief Return a QAbstractItemModel containing GPX files.
-     */
-    GpxFileModel  *gpxFileModel();
-
-    /**
-     * @brief Return a QAbstractItemModel containing files.
-     */
-    FileViewModel* fileViewModel() const;
-
     void clearPersistentTileCache();
     /**
      * @brief  Set the limit of the persistent (on hard disc) tile cache.
@@ -843,41 +899,22 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setDownloadUrl( const QUrl &url );
 
-    /**
-     * @brief Retrieve the map quality depending on the view context 
-     */
-    Marble::MapQuality mapQuality( Marble::ViewContext = Marble::Still );
-
     /**
      * @brief Set the map quality depending on the view context 
      */
     void setMapQuality( Marble::MapQuality, Marble::ViewContext = Marble::Still );
 
-    /**
-     * @brief Retrieve the view context (i.e. still or animated map) 
-     */
-    Marble::ViewContext viewContext( ) const;
-
     /**
      * @brief Set the view context (i.e. still or animated map) 
      */
     void setViewContext( Marble::ViewContext viewContext );
 
-    /**
-     * @brief Retrieve whether travels to a point should get animated 
-     */
-    bool animationsEnabled() const;
-
     /**
      * @brief Set whether travels to a point should get animated 
      */
     void setAnimationsEnabled( bool enabled );
 
-    Marble::AngleUnit defaultAngleUnit() const;
-    void setDefaultAngleUnit( Marble::AngleUnit angleUnit );
-
-    QFont defaultFont() const;
-    void setDefaultFont( const QFont& font );
+    void setSelection(const QRect& region);
 
  private Q_SLOTS:
 
@@ -902,11 +939,17 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
 
     void  mouseMoveGeoPosition( QString );
 
-    void  mouseClickGeoPosition( double lon, double lat, GeoDataPoint::Unit );
+    void  mouseClickGeoPosition( qreal lon, qreal lat, GeoDataCoordinates::Unit );
 
     void  timeout();
 
-    void  framesPerSecond( double fps );
+    void  framesPerSecond( qreal fps );
+
+    /** This signal is emit when a new rectangle region is selected over the map 
+     *  The list of double values include coordinates in degrees using this order:
+     *  lon1, lat1, lon2, lat2 (or West, North, East, South) as left/top, right/bottom rectangle.
+     */
+    void  regionSelected(const QList<double>&);
 
  protected:
     /**
@@ -939,5 +982,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     MarbleWidgetPrivate  * const d;
 };
 
+}
 
 #endif // MARBLEWIDGET_H
