@@ -16,12 +16,15 @@
 #include <QtCore/QDebug>
 #include <QtCore/QList>
 #include <QtCore/QPluginLoader>
+#include <QtCore/QTime>
 
 // Local dir
 #include "MarbleDirs.h"
-#include "MarbleAbstractLayer.h"
+#include "MarbleRenderPlugin.h"
 #include "MarbleAbstractFloatItem.h"
 
+namespace Marble
+{
 
 class PluginManagerPrivate
 {
@@ -30,7 +33,7 @@ class PluginManagerPrivate
     {
     }
 
-    QList<MarbleAbstractLayer *> m_layerPlugins;
+    QList<MarbleRenderPlugin *> m_renderPlugins;
 };
 
 PluginManager::PluginManager( QObject *parent )
@@ -38,13 +41,16 @@ PluginManager::PluginManager( QObject *parent )
       d( new PluginManagerPrivate() )
 {
     // For testing:
+    QTime t;
+    t.start();
     loadPlugins();
+    qDebug("Time elapsed: %d ms", t.elapsed());
 }
 
 PluginManager::~PluginManager()
 {
-//    qDeleteAll( d->m_layerPlugins );
-//    d->m_layerPlugins.clear();
+    qDeleteAll( d->m_renderPlugins );
+    d->m_renderPlugins.clear();
     delete d;
 }
 
@@ -52,8 +58,8 @@ QList<MarbleAbstractFloatItem *> PluginManager::floatItems() const
 {
     QList<MarbleAbstractFloatItem *> floatItemList;
 
-    QList<MarbleAbstractLayer *>::const_iterator i;
-    for (i = d->m_layerPlugins.begin(); i != d->m_layerPlugins.end(); ++i)
+    QList<MarbleRenderPlugin *>::const_iterator i;
+    for (i = d->m_renderPlugins.constBegin(); i != d->m_renderPlugins.constEnd(); ++i)
     {
         MarbleAbstractFloatItem *floatItem = qobject_cast<MarbleAbstractFloatItem *>(*i);
         if ( floatItem )
@@ -65,9 +71,9 @@ QList<MarbleAbstractFloatItem *> PluginManager::floatItems() const
     return floatItemList;
 }
 
-QList<MarbleAbstractLayer *> PluginManager::layerPlugins() const
+QList<MarbleRenderPlugin *> PluginManager::renderPlugins() const
 {
-    return d->m_layerPlugins;
+    return d->m_renderPlugins;
 }
 
 void PluginManager::loadPlugins()
@@ -79,25 +85,27 @@ void PluginManager::loadPlugins()
 
     MarbleDirs::debug();
 
-    qDeleteAll( d->m_layerPlugins );
-    d->m_layerPlugins.clear();
+    qDeleteAll( d->m_renderPlugins );
+    d->m_renderPlugins.clear();
 
     foreach( const QString &fileName, pluginFileNameList ) {
         qDebug() << fileName << " - " << MarbleDirs::pluginPath( fileName );
         QPluginLoader loader( MarbleDirs::pluginPath( fileName ) );
 
         QObject *obj = loader.instance();
-        MarbleAbstractLayer * layerPlugin = qobject_cast<MarbleAbstractLayer *>(obj);
+        MarbleRenderPlugin* layerPlugin = qobject_cast<MarbleRenderPlugin *>(obj);
+        MarbleRenderPlugin* instance = layerPlugin->instance();
 
-        if( layerPlugin ) {
-            d->m_layerPlugins.append( layerPlugin );
+        if( layerPlugin && instance ) {
+            d->m_renderPlugins.append( instance );
         }
-        else
-        {
+        else {
             qDebug() << "Plugin Failure: " << fileName << " is not a valid Marble Plugin:";
             qDebug() << loader.errorString();
         }
     }
 }
 
+}
+
 #include "PluginManager.moc"
