@@ -25,12 +25,18 @@
 
 #include "GeoTagHandler.h"
 
-class DgmlSectionTagHandler : public GeoTagHandler {
-public:
-    DgmlSectionTagHandler();
-    virtual ~DgmlSectionTagHandler();
+namespace Marble
+{
+namespace dgml
+{
 
+class DgmlSectionTagHandler : public GeoTagHandler
+{
+public:
     virtual GeoNode* parse(GeoParser&) const;
 };
 
+}
+}
+
 #endif // DgmlSectionTagHandler_h
