@@ -24,12 +24,18 @@
 
 #include "GeoTagHandler.h"
 
-class KmlaltitudeModeTagHandler : public GeoTagHandler {
-public:
-    KmlaltitudeModeTagHandler();
-    virtual ~KmlaltitudeModeTagHandler();
+namespace Marble
+{
+namespace kml
+{
 
+class KmlaltitudeModeTagHandler : public GeoTagHandler
+{
+public:
     virtual GeoNode* parse(GeoParser&) const;
 };
 
+}
+}
+
 #endif // KmlaltitudeModeTagHandler_h
