@@ -15,14 +15,17 @@
 #include "parleydocument.h"
 
 #include "../config-parley.h"
-#include "parley.h"
+#include "parleymainwindow.h"
+#include "editor/editor.h"
 #include "version.h"
 #include "prefs.h"
 
-#include "newdocument-wizard/kvtnewdocumentwizard.h"
 #include "vocabulary/vocabularyview.h"
+#include "settings/documentproperties.h"
+#include "welcomescreen/welcomescreen.h"
 
 #include <keduvoclesson.h>
+#include <keduvocleitnerbox.h>
 #include <keduvocexpression.h>
 #include <keduvocwordtype.h>
 
@@ -39,7 +42,7 @@
 #include <QtGui/QPrintDialog>
 
 #ifdef HAVE_LIBXSLT
-#include "export/export.h"
+#include "export/exportdialog.h"
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxslt/xslt.h>
@@ -49,7 +52,7 @@
 #endif
 
 
-ParleyDocument::ParleyDocument(ParleyApp *parent)
+ParleyDocument::ParleyDocument(ParleyMainWindow *parent)
  : QObject(parent)
 {
     m_parleyApp = parent;
@@ -61,7 +64,10 @@ ParleyDocument::ParleyDocument(ParleyApp *parent)
 
 ParleyDocument::~ParleyDocument()
 {
-    m_parleyApp->m_vocabularyView->saveColumnVisibility(m_doc->url());
+    if (m_doc) {
+        // FIXME: this should be part of the editor
+        m_parleyApp->editor()->m_vocabularyView->saveColumnVisibility(m_doc->url());
+    }
     delete m_doc;
 }
 
@@ -75,32 +81,59 @@ KEduVocDocument * ParleyDocument::document()
 void ParleyDocument::slotFileNew()
 {
     if (m_parleyApp->queryExit()) {
-        newDocumentWizard();
+        newDocument(true);
     }
 }
 
 
-void ParleyDocument::newDocument()
+void ParleyDocument::newDocument(bool wizard)
 {
-    disconnect(m_doc);
-    delete m_doc;
-    m_doc = new KEduVocDocument(this);
-    m_parleyApp->updateDocument();
-
-    initializeDefaultGrammar();
-    createExampleEntries();
-
-    m_doc->setModified(false);
-    m_parleyApp->updateDocument();
+    KEduVocDocument *newDoc = new KEduVocDocument();
+    bool setupLanguage = false;
+    initializeDefaultGrammar(newDoc);
+    createExampleEntries(newDoc);
+    if (wizard) {
+        DocumentProperties* titleAuthorWidget = new DocumentProperties(newDoc, true, m_parleyApp);
+        KDialog* titleAuthorDialog;
+        titleAuthorDialog = new KDialog(m_parleyApp);
+        titleAuthorDialog->setMainWidget( titleAuthorWidget );
+        titleAuthorDialog->setCaption(i18nc("@title:window document properties", "Properties for %1", newDoc->url().url()));
+        connect(titleAuthorDialog, SIGNAL(accepted()), titleAuthorWidget, SLOT(accept()));
+        if(titleAuthorDialog->exec()) {
+            setupLanguage = titleAuthorWidget->grammarCheckBox->isChecked();
+            delete titleAuthorDialog;
+        } else {
+            delete titleAuthorDialog;
+            return;
+        }
+    }
 
+    close();
+    m_doc = newDoc;
+    m_parleyApp->editor()->updateDocument();
     emit documentChanged(m_doc);
+
+    if(setupLanguage) {
+        m_parleyApp->editor()->slotLanguageProperties();
+    }
+
+    m_parleyApp->showEditor();
 }
 
 void ParleyDocument::slotFileOpen()
 {
     if (m_parleyApp->queryExit()) {
-        KUrl url = KFileDialog::getOpenUrl(QString(), KEduVocDocument::pattern(KEduVocDocument::Reading), m_parleyApp, i18n("Open Vocabulary Document"));
-        open(url, true);
+        QCheckBox *practiceCheckBox = new QCheckBox(i18n("Open in practice &mode"));
+        KFileDialog dialog(QString(), KEduVocDocument::pattern(KEduVocDocument::Reading), m_parleyApp, practiceCheckBox);
+        dialog.setCaption(i18n("Open Vocabulary Collection"));
+        if(dialog.exec() && !dialog.selectedUrl().isEmpty()) {
+            open(dialog.selectedUrl());
+            if(practiceCheckBox->isChecked()) {
+                m_parleyApp->startPractice();
+            } else {
+                m_parleyApp->showEditor();
+            }
+        }
     }
 }
 
@@ -108,39 +141,43 @@ void ParleyDocument::slotFileOpenRecent(const KUrl& url)
 {
     if (m_parleyApp->queryExit()) {
         open(url);
+        m_parleyApp->showEditor(); ///@todo: start practice directly depending on current component
     }
 }
 
-
-void ParleyDocument::open(const KUrl & url, bool addRecent)
+void ParleyDocument::open(const KUrl & url)
 {
     if (!url.path().isEmpty()) {
-        emit documentChanged(0);
-        disconnect(m_doc);
-        delete m_doc;
+        close();
         m_doc = new KEduVocDocument(this);
         m_doc->setCsvDelimiter(Prefs::separator());
         m_doc->open(url);
 
-        m_parleyApp->updateDocument();
+        m_parleyApp->editor()->updateDocument();
+        m_parleyApp->m_recentFilesAction->addUrl(url, m_doc->title());
 
-        if (addRecent) { // open sample does not go into recent
-            m_parleyApp->m_recentFilesAction->addUrl(url);
-        }
         emit documentChanged(m_doc);
     }
 }
 
-
-void ParleyDocument::openExample()
-{
-    if (m_parleyApp->queryExit()) {
-        QString s = KStandardDirs::locate("data", "parley/examples/");
-        KUrl url = KFileDialog::getOpenUrl(s, KEduVocDocument::pattern(KEduVocDocument::Reading), m_parleyApp, i18n("Open Example Vocabulary Document"));
-        open(url, false);
-    }
+void ParleyDocument::close() {
+    emit documentChanged(0);
+    disconnect(m_doc);
+    delete m_doc;
+    m_doc = 0;
 }
 
+void ParleyDocument::openGHNS()
+{
+    if (m_parleyApp->queryExit()) {
+        QString s = KStandardDirs::locateLocal("data", "kvtml/");
+        KUrl url = KFileDialog::getOpenUrl(s, KEduVocDocument::pattern(KEduVocDocument::Reading), m_parleyApp, i18n("Open Downloaded Vocabulary Collection"));
+        if (!url.isEmpty()) {
+            open(url);
+            m_parleyApp->showEditor();
+        }
+    }
+}
 
 void ParleyDocument::save()
 {
@@ -154,12 +191,12 @@ void ParleyDocument::save()
     }
 
     // remove previous backup
-    QFile::remove(QFile::encodeName(m_doc->url().path()+'~'));
+    QFile::remove(m_doc->url().path()+'~');
     ::rename(QFile::encodeName(m_doc->url().path()), QFile::encodeName(m_doc->url().path()+'~'));
 
     m_doc->setCsvDelimiter(Prefs::separator());
 
-    m_parleyApp->m_vocabularyView->saveColumnVisibility(m_doc->url());
+    m_parleyApp->editor()->m_vocabularyView->saveColumnVisibility(m_doc->url());
 
     int result = m_doc->saveAs(m_doc->url(), KEduVocDocument::Automatic, QString::fromLatin1("Parley ") + PARLEY_VERSION_STRING);
     if ( result != 0 ) {
@@ -167,7 +204,7 @@ void ParleyDocument::save()
         saveAs();
         return;
     }
-    m_parleyApp->m_recentFilesAction->addUrl(m_doc->url());
+    m_parleyApp->m_recentFilesAction->addUrl(m_doc->url(), m_doc->title());
 }
 
 
@@ -189,7 +226,7 @@ void ParleyDocument::saveAs(KUrl url)
                          QString msg = i18nc("@info:status saving a file", "Saving %1", url.path());
 //                 slotStatusMsg(msg);
 
-                         QFile::remove(QFile::encodeName(url.path()+'~')); // remove previous backup
+                         QFile::remove(url.path()+'~'); // remove previous backup
                              ::rename(QFile::encodeName(url.path()), QFile::encodeName(QString(url.path()+'~')));
 
                              m_doc->setCsvDelimiter(Prefs::separator());
@@ -198,13 +235,13 @@ void ParleyDocument::saveAs(KUrl url)
                                  url.setFileName(url.fileName() + QString::fromLatin1(".kvtml"));
                              }
 
-                            m_parleyApp->m_vocabularyView->saveColumnVisibility(url);
+                            m_parleyApp->editor()->m_vocabularyView->saveColumnVisibility(url);
 
                              int result = m_doc->saveAs(url, KEduVocDocument::Automatic, "Parley");
                              if (result != 0) {
                                  KMessageBox::error(m_parleyApp, i18n("Writing file \"%1\" resulted in an error: %2", m_doc->url().url(), m_doc->errorDescription(result)), i18n("Save File"));
                              } else {
-                                 m_parleyApp->m_recentFilesAction->addUrl(m_doc->url());
+                                 m_parleyApp->m_recentFilesAction->addUrl(m_doc->url(), m_doc->title());
                              }
                      }
     }
@@ -263,56 +300,37 @@ void ParleyDocument::slotSaveSelection()
     */
 }
 
-
-
-
-
-
-void ParleyDocument::newDocumentWizard()
+void ParleyDocument::initializeDefaultGrammar(KEduVocDocument *doc)
 {
-    newDocument();
-
-    KVTNewDocumentWizard *wizard;
-
-    wizard = new KVTNewDocumentWizard(m_doc, m_parleyApp);
-    if( !wizard->exec() == KDialog::Accepted ){
-        delete wizard;
-        return;
-    }
-    delete wizard;
-}
-
-void ParleyDocument::initializeDefaultGrammar()
-{
-    KEduVocWordType *root = m_doc->wordTypeContainer();
+    KEduVocWordType *root = doc->wordTypeContainer();
     KEduVocWordType *noun = new KEduVocWordType(i18n("Noun"), root);
-    noun->setWordType(KEduVocWordType::Noun);
+    noun->setWordType(KEduVocWordFlag::Noun);
     root->appendChildContainer(noun);
 
-    KEduVocWordType *nounChild = new KEduVocWordType(i18n("Male"), noun);
-    nounChild->setWordType(KEduVocWordType::NounMale);
+    KEduVocWordType *nounChild = new KEduVocWordType(i18n("Masculine"), noun);
+    nounChild->setWordType(KEduVocWordFlag::Noun | KEduVocWordFlag::Masculine);
     noun->appendChildContainer(nounChild);
-    nounChild = new KEduVocWordType(i18n("Female"), noun);
-    nounChild->setWordType(KEduVocWordType::NounFemale);
+    nounChild = new KEduVocWordType(i18n("Feminine"), noun);
+    nounChild->setWordType(KEduVocWordFlag::Noun | KEduVocWordFlag::Feminine);
     noun->appendChildContainer(nounChild);
-    nounChild = new KEduVocWordType(i18n("Neutral"), noun);
-    nounChild->setWordType(KEduVocWordType::NounNeutral);
+    nounChild = new KEduVocWordType(i18n("Neuter"), noun);
+    nounChild->setWordType(KEduVocWordFlag::Noun | KEduVocWordFlag::Neuter);
     noun->appendChildContainer(nounChild);
 
     KEduVocWordType *verb = new KEduVocWordType(i18n("Verb"), root);
-    verb->setWordType(KEduVocWordType::Verb);
+    verb->setWordType(KEduVocWordFlag::Verb);
     root->appendChildContainer(verb);
 
     KEduVocWordType *adjective = new KEduVocWordType(i18n("Adjective"), root);
-    adjective->setWordType(KEduVocWordType::Adjective);
+    adjective->setWordType(KEduVocWordFlag::Adjective);
     root->appendChildContainer(adjective);
 
     KEduVocWordType *adverb = new KEduVocWordType(i18n("Adverb"), root);
-    adverb->setWordType(KEduVocWordType::Adverb);
+    adverb->setWordType(KEduVocWordFlag::Adverb);
     root->appendChildContainer(adverb);
 }
 
-void ParleyDocument::createExampleEntries()
+void ParleyDocument::createExampleEntries(KEduVocDocument *doc)
 {
     // some default values
     KUser user;
@@ -320,35 +338,43 @@ void ParleyDocument::createExampleEntries()
     if ( userName.isEmpty() ) {
         userName = user.loginName();
     }
-    m_doc->setAuthor( userName );
-    m_doc->setTitle( i18n("Welcome") );
-    m_doc->setLicense( i18n("Public Domain") );
-    m_doc->setCategory( i18n("Languages") );
+    doc->setAuthor( userName );
+    doc->setLicense( i18n("Public Domain") );
+    doc->setCategory( i18n("Languages") );
 
     QString locale = KGlobal::locale()->language();
 
-    m_doc->appendIdentifier();
-    m_doc->appendIdentifier();
-    m_doc->identifier(0).setName( KGlobal::locale()->languageCodeToName( locale) );
-    m_doc->identifier(0).setLocale( locale );
-    m_doc->identifier(1).setName( i18n("A Second Language") );
-    m_doc->identifier(1).setLocale( locale );
+    doc->appendIdentifier();
+    doc->appendIdentifier();
+    doc->identifier(0).setName( KGlobal::locale()->languageCodeToName( locale) );
+    doc->identifier(0).setLocale( locale );
+    doc->identifier(1).setName( i18n("A Second Language") );
+    doc->identifier(1).setLocale( locale );
 
-    KEduVocLesson* lesson = new KEduVocLesson(i18n("Lesson 1"), m_doc->lesson());
-    m_doc->lesson()->appendChildContainer(lesson);
+    KEduVocLesson* lesson = new KEduVocLesson(i18n("Lesson 1"), doc->lesson());
+    doc->lesson()->appendChildContainer(lesson);
+
+    KEduVocLeitnerBox *box = doc->leitnerContainer();
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 7 (best)"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 6"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 5"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 4"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 3"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 2"), box));
+    box->appendChildContainer(new KEduVocLesson(i18n("Box 1 (lowest)"), box));
 
     // add some entries
     for ( int i = 0; i < 15 ; i++ ) {
         lesson->appendEntry(new KEduVocExpression());
     }
 
-    m_doc->setModified(false);
+    doc->setModified(false);
 }
 
-
 void ParleyDocument::slotGHNS()
 {
     KNS::Entry::List entries = KNS::Engine::download();
+    int numberInstalled = 0;
     // list of changed entries
     foreach(KNS::Entry* entry, entries) {
         // care only about installed ones
@@ -358,22 +384,29 @@ void ParleyDocument::slotGHNS()
                 KMimeType::Ptr mimeType = KMimeType::findByPath(file);
                 kDebug() << "KNS2 file of mime type:" << KMimeType::findByPath(file)->name();
                 if (mimeType->name() == "application/x-kvtml") {
-                    KProcess newParley;
-                    newParley.setProgram("parley", QStringList() << file);
-                    newParley.startDetached();
-                    m_parleyApp->m_downloadedFilesAction->addUrl(file);
-                }
+                    m_parleyApp->m_recentFilesAction->addUrl(file);
+                    ++numberInstalled;
+                }         
             }
         }
     }
     qDeleteAll(entries);
+    // to enable the display in the welcome screen
+    m_parleyApp->m_recentFilesAction->saveEntries(KGlobal::config()->group("Recent Files"));
+    Prefs::self()->writeConfig();
+    m_parleyApp->m_welcomeScreen->updateRecentFilesModel();
+    if (numberInstalled > 1) {
+        openGHNS();
+    } else if (numberInstalled == 1) {
+        m_parleyApp->m_recentFilesAction->action(0)->trigger();
+    }
 }
 
-void ParleyDocument::exportHtmlDialog()
+void ParleyDocument::exportDialog()
 {
 #ifdef HAVE_LIBXSLT
-
-    ParleyExport::exportDocument(this, m_parleyApp);
+    ExportDialog dialog(this, m_parleyApp);
+    dialog.exec();
 #endif
 }
 
@@ -392,7 +425,7 @@ void ParleyDocument::slotFileMerge()
     //
 //         m_doc->merge(new_doc, true);
     //
-//         KEduVocConjugation::setTenseNames(m_doc->tenseDescriptions());
+//         KEduVocWordFlag::setTenseNames(m_doc->tenseDescriptions());
 //         KVTUsage::setUsageNames(m_doc->usageDescriptions());
     //
 //         delete(new_doc);
@@ -418,5 +451,4 @@ void ParleyDocument::enableAutoBackup(bool enable)
     }
 }
 
-
 #include "parleydocument.moc"
