@@ -22,9 +22,9 @@
 #include "vocabularydelegate.h"
 #include "vocabularymimedata.h"
 
-#include "parley.h"
+#include "editor/editor.h"
 #include "prefs.h"
-
+#include "vocabularycolumnsdialog.h"
 #include "documentsettings.h"
 
 #include <keduvoctranslation.h>
@@ -40,7 +40,6 @@
 #include <KComponentData>
 #include <KGlobalSettings>
 #include <KAction>
-#include <KActionMenu>
 #include <KActionCollection>
 #include <KToggleAction>
 #include <KLocale>
@@ -48,16 +47,11 @@
 #include <KUrl>
 #include <sonnet/backgroundchecker.h>
 
-VocabularyView::VocabularyView(ParleyApp * parent)
-    : QTableView(parent)
+VocabularyView::VocabularyView(Editor * parent)
+    : QTableView(parent), m_model(0), m_doc(0),
+    spellcheckRow(0), spellcheckColumn(0), spellingChecker(0), spellingDialog(0)
 {
-    m_model = 0;
-    m_doc = 0;
-
-    spellingChecker = 0;
-    spellingDialog = 0;
-    spellcheckRow = 0;
-    spellcheckColumn = 0;
+    installEventFilter(this);
 
     setHorizontalHeader(new VocabularyHeaderView(Qt::Horizontal, this));
 
@@ -65,7 +59,6 @@ VocabularyView::VocabularyView(ParleyApp * parent)
     setEditTriggers(QAbstractItemView::AnyKeyPressed | QAbstractItemView::EditKeyPressed | QAbstractItemView::DoubleClicked);
 
     setSortingEnabled(true);
-
     setTabKeyNavigation(true);
 
     m_vocabularyDelegate = new VocabularyDelegate(this);
@@ -84,9 +77,6 @@ VocabularyView::VocabularyView(ParleyApp * parent)
     // smooth scrolling horizontally, otherwise it tries to jump from item to item.
     setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
 
-    // create actions
-    m_vocabularyColumnsActionMenu = new KActionMenu(this);
-
     m_appendEntryAction = new KAction(this);
     parent->actionCollection()->addAction("edit_append", m_appendEntryAction);
     m_appendEntryAction->setIcon(KIcon("list-add-card"));
@@ -140,6 +130,18 @@ VocabularyView::VocabularyView(ParleyApp * parent)
     m_clearSelectionAction->setWhatsThis(i18n("Deselect all rows"));
     m_clearSelectionAction->setToolTip(m_clearSelectionAction->whatsThis());
     m_clearSelectionAction->setStatusTip(m_clearSelectionAction->whatsThis());
+
+    // vocabulary columns dialog
+    KAction *vocabularyColumnsDialogAction = new KAction(this);
+    parent->actionCollection()->addAction("show_vocabulary_columns_dialog", vocabularyColumnsDialogAction);
+    vocabularyColumnsDialogAction->setIcon(KIcon("view-file-columns"));
+    vocabularyColumnsDialogAction->setText(i18n("Vocabulary Columns")); ///@todo add "..." since this opens a dialog
+    vocabularyColumnsDialogAction->setWhatsThis(i18n("Toggle display of individual vocabulary columns"));
+    vocabularyColumnsDialogAction->setToolTip(vocabularyColumnsDialogAction->whatsThis());
+    vocabularyColumnsDialogAction->setStatusTip(vocabularyColumnsDialogAction->whatsThis());
+    horizontalHeader()->addAction(vocabularyColumnsDialogAction);
+    addAction(vocabularyColumnsDialogAction);
+    connect(vocabularyColumnsDialogAction, SIGNAL(triggered(bool)), this, SLOT(slotShowVocabularyColumnsDialog()));
 }
 
 
@@ -151,9 +153,6 @@ void VocabularyView::setModel(VocabularyFilter * model)
 
     connect(selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), SLOT(slotSelectionChanged(const QItemSelection&, const QItemSelection&)));
     slotSelectionChanged(QItemSelection(), QItemSelection());
-    
-    // for the header view workaround
-    connect(m_model, SIGNAL(rowsInserted(const QModelIndex&, int, int)), this, SLOT(headerViewWorkaround()));
 }
 
 // void VocabularyView::print(QPrinter * pPrinter)
@@ -163,61 +162,61 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //     painter.setBrush(Qt::NoBrush);
 //     int res = pPrinter->resolution();
 //     int marg = res;
-// 
+//
 //     int pw;
 //     int c = 0;
 //     int currentWidth;
 //     int startCol;
 //     int pageNum = 1;
-// 
+//
 //     int colCount = model()->columnCount(QModelIndex());
 //     int hh = horizontalHeader()->height();
 //     int tPos = marg + hh;
-// 
+//
 //     QStyleOptionViewItem option;
 //     option.initFrom(this);
-// 
+//
 //     painter.begin(pPrinter);
 //     QRect w = painter.window();
 //     QRect cr;
-// 
+//
 //     pw = w.width() - (2 * marg);
-// 
+//
 //     while (c < colCount) {
 //         startCol = c;
 //         currentWidth = marg;
-// 
+//
 //         while ((currentWidth < pw) && (c < colCount)) {
 //             currentWidth = currentWidth + columnWidth(c);
 //             c++;
 //         }
 //         if (c < colCount)
 //             c--;
-// 
+//
 //         newPage(painter, res, startCol, c);
-// 
+//
 //         for (int rc = 0; rc < model()->rowCount(QModelIndex()); ++rc) {
 //             int rh = rowHeight(rc);
-// 
+//
 //             painter.resetMatrix();
 //             painter.setFont(Prefs::tableFont());
 //             painter.translate(marg, tPos);
-// 
+//
 //             for (int i = startCol; i <= c && i < colCount; ++i) {
 //                 cr.setRect(0, 0, columnWidth(i), rh);
 //                 option.displayAlignment = Qt::AlignLeft | Qt::AlignVCenter;
 //                 option.rect = cr;
 //                 painter.drawRect(cr);
 //                 painter.save();
-// 
+//
 //                 itemDelegate()->paint(&painter, option, model()->index(rc, i));
 //                 painter.restore();
-// 
+//
 //                 painter.translate(columnWidth(i), 0);
 //             }
-// 
+//
 //             tPos = tPos + rowHeight(rc);
-// 
+//
 //             if (tPos + rowHeight(rc + 1) > w.height() - marg) {
 //                 endOfPage(painter, pageNum++, res);
 //                 tPos = marg + hh;
@@ -226,7 +225,7 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //             }
 //         }
 //         endOfPage(painter, pageNum++, res);
-// 
+//
 //         if (c < colCount) {
 //             pPrinter->newPage();
 //             tPos = marg + hh;
@@ -235,7 +234,7 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //     }
 //     painter.end();
 // }
-// 
+//
 // void VocabularyView::newPage(QPainter & painter, int res, int startCol, int endCol)
 // {
 //     int marg = res;
@@ -257,7 +256,7 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //         painter.translate(cw, 0);
 //     }
 // }
-// 
+//
 // void VocabularyView::endOfPage(QPainter & painter, int pageNum, int res)
 // {
 //     painter.resetMatrix();
@@ -266,28 +265,28 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //     QRect r = painter.boundingRect(0, 0, 0, 0, Qt::AlignLeft, QString::number(pageNum));
 //     painter.drawText((w.width()/2) - (r.width()/2), w.height() - res + 20, QString::number(pageNum));
 // }
-// 
+//
 // void VocabularyView::resizeEvent(QResizeEvent * event)
 // {
 //     QWidget::resizeEvent(event);
-// 
+//
 // //     if (event == 0) {
 // //         return;
 // //     }
-// 
+//
 //     if ( !model() || (model()->columnCount() < 3) ) {
 //         return;
 //     }
-// 
+//
 //     QHeaderView * header = horizontalHeader();
 //     int colCount =  model()->columnCount(QModelIndex());
 //     int oldWidth = 0;
-// 
+//
 //     for (int i = 0; i < colCount; ++i)
 //         oldWidth += header->sectionSize(i);
-// 
+//
 //     int newWidth = viewport()->width();
-// 
+//
 //     switch (Prefs::headerResizeMode()) {
 //     case Prefs::EnumHeaderResizeMode::Automatic: {
 //             // lesson is only half as wide as a original/translation
@@ -297,31 +296,31 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //                 newWidth -= KV_COLWIDTH_MARK;
 //                 header->resizeSection(KV_COL_MARK, KV_COLWIDTH_MARK);
 //             }
-// 
+//
 //             int columnsHalfWidth;
 //             if ( Prefs::tableLessonColumnVisible() ) {
-//                 // total width / ((total number of columns - active column) * 2 -1 lesson only half) 
+//                 // total width / ((total number of columns - active column) * 2 -1 lesson only half)
 //                 columnsHalfWidth = newWidth / ((colCount - 1) * 2 -1);
 //                 header->resizeSection(KV_COL_LESS, columnsHalfWidth);
 //             } else {
 //                 columnsHalfWidth = newWidth / ((colCount - 2) * 2);
 //             }
-// 
+//
 //             for (int currentColumn = KV_COL_TRANS; currentColumn < colCount; currentColumn++) {
 //                 header->resizeSection(currentColumn, 2 * columnsHalfWidth);
 //             }
 //         }
 //         break;
-// 
+//
 //     case Prefs::EnumHeaderResizeMode::Percent: {
 //             float grow = float(newWidth) / float(oldWidth);
 //             header->resizeSection(KV_COL_MARK, KV_COLWIDTH_MARK);
-// 
+//
 //             int remainder = newWidth - KV_COLWIDTH_MARK;
 //             int x = qMax(HEADER_MINSIZE, (int)((header->sectionSize(KV_COL_LESS) * grow) + 0.5));
 //             header->resizeSection(KV_COL_LESS, x);
 //             remainder -= x;
-// 
+//
 //             for (int i = KV_COL_TRANS; i < colCount - 1; i++) {
 //                 x = qMax(HEADER_MINSIZE, (int)((header->sectionSize(i) * grow) + 0.5));
 //                 remainder -= x;
@@ -330,13 +329,13 @@ void VocabularyView::setModel(VocabularyFilter * model)
 //             header->resizeSection(colCount - 1, qMax(HEADER_MINSIZE, remainder));
 //         }
 //         break;
-// 
+//
 //     case Prefs::EnumHeaderResizeMode::Fixed:
 //         // do nothing
 //         break;
 //     }
 // }
-// 
+//
 // void VocabularyView::showEvent(QShowEvent * event)
 // {
 //     QWidget::showEvent(event);
@@ -357,12 +356,9 @@ void VocabularyView::slotCurrentChanged(const QModelIndex & current, const QMode
 void VocabularyView::reset()
 {
     QTableView::reset();
+    emit translationChanged(0, 0);
+
     kDebug() << "reset";
-///@todo check if the actions are recreated every time when selecting a diff lesson.
-    foreach( KAction* oldAction, m_columnActionMap.keys() ) {
-        m_columnActionMap.remove(oldAction);
-        delete oldAction;
-    }
 
     QList<int> visibleColumns;
     if (m_doc) {
@@ -379,57 +375,27 @@ void VocabularyView::reset()
         }
     }
 
-    KActionMenu* currentTranslationAction = 0;
     for( int i = 0; i < model()->columnCount(QModelIndex()); i++) {
-        KAction* columnAction;
-
-        if(VocabularyModel::columnType(i) == VocabularyModel::Translation) {
-            currentTranslationAction = new KActionMenu(
-                model()->headerData(i, Qt::Horizontal).toString(), this);
-            columnAction = currentTranslationAction;
-            m_vocabularyColumnsActionMenu->addAction(currentTranslationAction);
-        } else {
-            if (currentTranslationAction) {
-                columnAction = new KToggleAction(model()->headerData(i, Qt::Horizontal).toString(), this);
-                currentTranslationAction->addAction(columnAction);
-                connect (columnAction, SIGNAL(triggered(bool)),
-                    this, SLOT(slotToggleColumn(bool)));
-                if (i < visibleColumns.size() && visibleColumns.value(i) == 1) {
-                    // show the column
-                    columnAction->setChecked(true);
-                    setColumnHidden(i, false);
-                    // for the header view workaround
-                    m_workaroundColumnVisibilityMap[i] = true;
-                } else {
-                    columnAction->setChecked(false);
-                    setColumnHidden(i, true);
-                    // for the header view workaround
-                    m_workaroundColumnVisibilityMap[i] = false;
-                }
-            }
+        if (i < visibleColumns.size()) {
+            setColumnHidden(i, !visibleColumns.value(i));
         }
-        m_columnActionMap[columnAction] = i;
     }
 }
 
-void VocabularyView::slotToggleColumn(bool show)
-{
-    setColumnHidden(m_columnActionMap[(KAction*)sender()], !show);
-    // for the header view workaround
-    m_workaroundColumnVisibilityMap[m_columnActionMap[(KAction*)sender()]] = show;
-}
-
 void VocabularyView::saveColumnVisibility(const KUrl & kurl) const
 {
+    if(!m_doc) {
+        return;
+    }
     // Generate a QList<int> for saving
-    QList<int> qli;
-    for (int i = 0; i < m_columnActionMap.size(); ++i)
+    QList<int> visibleList;
+    for (int i = 0; i < m_model->columnCount(); ++i)
     {
-        qli.append(static_cast<int>(!isColumnHidden(i)));
+        visibleList.append(static_cast<int>(!isColumnHidden(i)));
     }
 
     DocumentSettings ds(kurl.url());
-    ds.setVisibleColumns(qli);
+    ds.setVisibleColumns(visibleList);
     ds.writeConfig();
 
     QByteArray saveState = horizontalHeader()->saveState();
@@ -489,7 +455,7 @@ void VocabularyView::slotEditPaste()
     const QMimeData *mimeData = clipboard->mimeData();
     const VocabularyMimeData *vocMimeData = qobject_cast<const VocabularyMimeData *>(mimeData);
     if (vocMimeData) {
-        kDebug() << "clipboard contains vocabulary mime data!";
+        kDebug() << "Clipboard contains vocabulary mime data.";
         foreach(const VocabularyMimeData::MimeExpression &mimeEntry, vocMimeData->expressionList()) {
             KEduVocExpression *pasteExpression = new KEduVocExpression(mimeEntry.expression);
             m_model->appendEntry(pasteExpression);
@@ -511,53 +477,21 @@ void VocabularyView::slotEditPaste()
                 pasteExpression->translation(translation)->setWordType(type);
                 // check for special type stuff
                 if (type->wordType() != mimeEntry.wordTypes.value(translation).grammarType) {
-                    if (type->wordType() == KEduVocWordType::General) {
+                    if (type->wordType() == KEduVocWordFlag::NoInformation) {
                         type->setWordType(mimeEntry.wordTypes.value(translation).grammarType);
                     }
                 }
             }
         }
-    } else {
-        m_model->appendEntry(new KEduVocExpression(mimeData->text()));
-        kDebug() << "clipboard contains text data!";
+    } else if (mimeData->hasText()) {
+        kDebug() << "Clipboard contains text data.";
+        // split at newline
+        QStringList lines = clipboard->text().split('\n');
+        foreach(QString line, lines) {
+            // split at tabs or semicolon:
+            m_model->appendEntry(new KEduVocExpression(line.split(QRegExp("[\t;]"), QString::KeepEmptyParts)));
+        }
     }
-
-    /// @todo make the pasted stuff visible by making the corresponding lesson visible, if it is not (?)
-//     slotStatusMsg(i18n("Inserting clipboard contents..."));
-
-/*    QApplication::setOverrideCursor(Qt::WaitCursor);
-    QString s;
-    QString textToPaste = QApplication::clipboard()->text();
-
-    QTextStream ts;
-    ts.setString(&textToPaste, QIODevice::Text);
-
-    QString num;
-
-    QModelIndexList selectedRows = m_tableView->selectionModel()->selectedRows();
-    int lastSelectedRow;
-    if(!selectedRows.isEmpty())
-    lastSelectedRow = m_sortFilterModel->mapToSource(selectedRows.back()).row() + 1;
-    else
-    lastSelectedRow = m_tableModel->rowCount(QModelIndex());
-
-    int count = 0;
-    while (!ts.atEnd()) {
-    s = ts.readLine();
-    if (!s.isEmpty()) {
-    m_tableModel->insertRows(lastSelectedRow + count, 1);
-    QStringList sl = s.split('\t', QString::KeepEmptyParts);
-
-    for (int i = 0; i < sl.count(); ++i) {
-    m_tableModel->setData(m_tableModel->index(lastSelectedRow + count, i + KV_COL_TRANS), sl[i], Qt::EditRole);
-//                 m_tableModel->setData(m_tableModel->index(lastSelectedRow + count, i + KV_COL_TRANS), m_document->document()->currentLesson(), KVTTableModel::LessonRole);
-    m_tableModel->setData(m_tableModel->index(lastSelectedRow + count, i + KV_COL_TRANS), m_lessonDockWidget->selectedLesson(), KVTTableModel::LessonRole);
-}
-}
-    count++;
-}
-
-    m_deleteEntriesAction->setEnabled(m_sortFilterModel->rowCount(QModelIndex()) > 0);*/
 }
 
 void VocabularyView::slotCutEntry()
@@ -566,11 +500,6 @@ void VocabularyView::slotCutEntry()
     deleteSelectedEntries(false);
 }
 
-KActionMenu * VocabularyView::columnsActionMenu()
-{
-    return m_vocabularyColumnsActionMenu;
-}
-
 void VocabularyView::slotSelectionChanged(const QItemSelection &, const QItemSelection &)
 {
     bool hasSelection = selectionModel()->hasSelection();
@@ -587,6 +516,16 @@ void VocabularyView::setDocument(KEduVocDocument * doc)
     m_vocabularyDelegate->setDocument(doc);
 }
 
+/**
+ * Set the translator to be used by the delegate
+ * @param translator
+ */
+void VocabularyView::setTranslator(Translator* translator)
+{
+    m_vocabularyDelegate->setTranslator(translator);
+}
+
+
 void VocabularyView::checkSpelling()
 {
     // values so we will start in the first row:
@@ -595,6 +534,15 @@ void VocabularyView::checkSpelling()
     continueSpelling();
 }
 
+void VocabularyView::slotShowVocabularyColumnsDialog()
+{
+    VocabularyColumnsDialog *dialog = new VocabularyColumnsDialog(m_doc, this);
+
+    if ( dialog->exec() == KDialog::Accepted ) {
+        reset();
+    }
+}
+
 void VocabularyView::continueSpelling()
 {
     if (spellcheckRow >= m_model->rowCount()) {
@@ -661,6 +609,19 @@ void VocabularyView::selectIndex(const QModelIndex &newIndex)
     scrollTo(newIndex);
 }
 
+bool VocabularyView::eventFilter(QObject* obj, QEvent* event) {
+    if (event->type() == QEvent::KeyPress && Prefs::smartAppend()) {
+        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
+        if (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) {
+            if (selectionModel()->currentIndex().row() == m_model->rowCount() - 1) {
+                m_model->appendEntry();
+            }
+        }
+    }
+    // standard event processing
+    return QObject::eventFilter(obj, event);
+}
+
 void VocabularyView::misspelling(const QString & word, int start)
 {
     QModelIndex index = m_model->index(spellcheckRow, spellcheckColumn, QModelIndex());
@@ -677,25 +638,9 @@ void VocabularyView::spellingReplace(const QString & oldWord, int start, const Q
     m_model->setData(index, newData);
 }
 
-
-// This slot is used as a temporary workaround for a Qt bug:
-// http://trolltech.com/developer/task-tracker/index_html?method=entry&id=212660
-// The bug causes all columns to be shown when you accidentally click on the header view
-// when the vocabulary view is empty, which can be really annoying.
-//
-// To minimize the impact of this bug, a map of visible columns is stored and applied every
-// time a row is inserted, so that the accidentally shown columns are removed again after
-// showing come entries.
-//
-// This code should be removed once the bug in Qt is fixed (probably 4.4.2).
-void VocabularyView::headerViewWorkaround()
+QModelIndexList VocabularyView::getSelectedIndexes() const 
 {
-    QMapIterator<int, bool> i(m_workaroundColumnVisibilityMap);
-    while (i.hasNext()) {
-        i.next();
-        setColumnHidden(i.key(), !i.value());
-    }
+    return selectionModel()->selectedIndexes();
 }
 
 #include "vocabularyview.moc"
-
